;==============================================================
; CASEVAC - main program
;
; Version 1.0
; 2016
; Open source (please feel free to use this code as you wish)
;
; Built with WLA DX assembler and linker
; Maxim's BMP2TILE utility used to generate tile data entries
;
; * This file contains the main executeable code of the program.
; * This file also contains notes regarding game development
;   and overall software architecture/design
;
; Concept, art and programming by Patrick Beeson
;==============================================================

; Game concept
; ------------

; CASEVAC is a top down, map based game where the player commands the movements
; of a combat ambulance (graphics loosely based on a M113 Amb). The objective of
; the game is to collect ten casualties while avoiding damage from IEDs, falling
; artillery rounds and RPGs. The player starts on the home map (0-1) just outside of a
; field hospital. The player then departs on patrol and may move between the following
; maps:

; |-----|-----|-----|-----|-----|-----|
; | 0-0 | 1-0 | 2-0 | 3-0 | 4-0 | 5-0 |
; |-----|-----|-----|-----|-----|-----|
; | 0-1 | 1-1 | 2-1 | 3-1 | 4-1 | 5-1 |
; |-----|-----|-----|-----|-----|-----|
; | 0-2 | 1-2 | 2-2 | 3-2 | 4-2 | 5-2 |
; |-----|-----|-----|-----|-----|-----|

; The numbers in the diagram are referred to as map indexes. The leading index
; is for the x axis while the lagging index is for the y axis. The player moves
; the ambulence with the game pad direction buttons. In response, the ambulence moves around
; the screen as its sprite's x and y coordinates change. The player moves from one
; map to another by crossing the boarder between maps; these boarders may be
; found at either the top, bottom, left or right edges of the screen.
; The game then automatically draws the new map and places the ambulance
; in the appropriate position. The world is designed to be finite but unbounded.
; That is to say, the player can move in any direction
; without ever reaching the end of the world; the player simply wraps from one
; side of the world to the other when they encounter "the end." It should be noted that
; the player is not alone in the world; mobs (mobile game entities) are present and
; represent threats. These mobs include IEDs (not mobile per se but can harm the ambulence),
; falling artillery rounds and flying RPG rounds.

; Graphics
; --------

; CASEVAC is a game designed for the Sega Master System. This
; hardware platform constrains how the graphics of the game are constructed.
; All the graphics are composed of 8 x 8 bitmaps. These bitmaps may be drawn on the
; background - in which case they are tiles - or the foreground - in which case
; they are sprites. In this game, tiles are immobile while sprites may or may not
; move. The maps alluded to above consist of a matrix of tile indexes (see data.asm).
; Sprites are used to display casualties, IEDs, Arty rounds and RPG rounds. Sprite collision detection
; is implemented by drawing upon the VDP's ability to send sprite collision signals
; via the VDPControl port.

; Overall design architecture
; ---------------------------

; The following is the overall flow of game execution:

; N.B. Not all code from the main function is included in the diagram which follows;
; only the high level components of the software architecture are included. It should also
; be noted that sound effects have been added for CASEVAC; these are implemented as sound
; effects function calls from a variety of places within the software as required to que
; sounds prn.

;                     ----------------
;                     | Boot section |
;                     ----------------
;                         |
;                         V
;                     -----------------
;                     | Main function |
;                     -----------------
;                         |
;                         -----
;                             |
;                             V
;                         ------------------
;                         | Initialize VDP |---------------------------------> IniVDP function
;                         ------------------
;                             |
;                             V
;                         -------------------------------------------
;                         | Draw title screen and wait for keypress |--------> DrawTitleScreen and WaitForKeyPress functions
;                         -------------------------------------------
;                             |
;                             V
;                         -------------------------------------------
;                         | Draw intro screen and wait for keypress |--------> DrawIntroScreen and WaitForKeyPress functions
;                         -------------------------------------------
;                             |
;                             V
;                         ------------------------------
;                         | Initialize variable values |---------------------> InitializeVariables function
;                         ------------------------------
;                             |
;                             V
;                         -------------------
;                   ----->| Main event loop |
;                   |     -------------------
;                   |         |
;                   |         -----
;                   |             |
;                   |             V
;                   |         -----------------------------
;                   |         | Get player control inputs |------------------> GetInputs function
;                   |         -----------------------------
;                   |             |
;                   |             V
;                   |         ---------------------------------------
;                   |         | Update ambulence position variables |--------> MovePlayer function
;                   |         ---------------------------------------
;                   |             |
;                   |             V
;                   |         -------------------------------------
;                   |         | Update map background if required |----------> UpdateTiles function
;                   |         -------------------------------------
;                   |             |
;                   |             V
;                   |         ----------------------------------------------
;                   |         | Update lives and casualties indicator bars |-> DrawLivesAndCas function
;                   |         ----------------------------------------------
;                   |             |
;                   |             V
;                   |         ---------------------------------------------
;                   |         | Casualty collision detection and event    |--> Implemented as a portion of the
;                   |         | handling                                  |    CollisionHandler function
;                   |         | i.e.: 1) remove casualty from screen      |
;                   |         |       2) increment casualty score         |
;                   |         |       3) trigger win screen if player has |
;                   |         |          collected all 10 casualties      |
;                   |         ---------------------------------------------
;                   |             |
;                   |             V
;                   |         ----------------------------------------
;                   |         | Enemy collision detection and event  |-------> Implemented as a portion of the
;                   |         | handling                             |         CollisionHandler function
;                   |         | i.e.: 1) remove enemy from screen    |
;                   |         |       2) trigger blast graphic       |
;                   |         |       3) decrement player's lives    |
;                   |         |       4) trigger game over screen if |
;                   |         |          player runs out of lives    |
;                   |         ----------------------------------------
;                   |             |
;                   |             V
;                   |         --------------------------------------
;                   |         | Update sprites displayed on screen |---------> UpdateSprites function
;                   |         | and move mobs                      |
;                   |         --------------------------------------
;                   |             |
;                   |             V
;                   |         ----------------------------------
;                   ----------| Jump to top of main event loop |
;                             ----------------------------------

; As previously mentioned, the game world is made up of background (tile) elements and
; foreground (sprite) elements. Sprites are used for all entities with which the player
; may interact. There are two sprites to represent the player's amulance (1 for the front and 1 for the back);
; there are sprites which represent the casulaties the player has to pickup; finally, there are sprites
; which represent enemies (IEDs, Arty rounds and RPG rounds. Interactions between the character and the other
; sprites is governed by VDP sprite collision detection and those software routines
; triggered by these collisions. From an implementation standpoint, sprites are controlled
; by objects of type Sprite (see memory_map.asm). The data within these objects is used to
; govern their drawing and the events which occur upon collision detection.

; Testing procedures conducted
; ----------------------------

; 1) Many components recycled from a prototype game I designed. These components were
;    tested as part of the previous game's development process.
; 2) Components tested as they were designed/built using Kega Fusion 3.64 (Windows), Meka 0.73 (Windows)
;    and MasterEmu 2.3 (Android)

; Known bugs and deficiencies
; ---------------------------

; 1) Impassable object collision detection leads to player stopping 1 tile prior to impact
;    when travelling upwards.
; 2) Impassable collision detection when moving left into a wall prevents movement
;    up or down the wall until player moves backwards
; 3) Occassionally, player's transitions from map to map do not work correctly.
;    Sometimes they wrap to the opposite side of the current map instead of
;    changing maps.
; 4) Pressing pause causes tile drawing issues whereby blank tiles are changed to
;    letter tiles
; 5) Sprite update algorthim requires that 1 object of each class be placed in map 0-0.
;    Otherwise, program execution slows down. The problem is that map 0-0 is supposed to
;    be close to the start position of the player and therefore somewhat safe.
;    The solution to this bug is to shift maps as follows with the player starting on 1-1:

; |-----|-----|-----|-----|-----|-----|
; | 1-0 | 2-0 | 3-0 | 4-0 | 5-0 | 0-0 |
; |-----|-----|-----|-----|-----|-----|
; | 1-1 | 2-1 | 3-1 | 4-1 | 5-1 | 0-1 |
; |-----|-----|-----|-----|-----|-----|
; | 1-2 | 2-2 | 3-2 | 4-2 | 5-2 | 0-2 |
; |-----|-----|-----|-----|-----|-----|

;    This configuration allows the player to move to progressively more difficult maps as
;    they move right.
; 6) Generally inefficient use of ROM memory for storage of game data
; 7) Some disorganized procedural decomposition
; 8) When two or more RPG objects are on a single map their homing algorithms mutually interfer
;    causing some to home and other to not.
; 9) The pitch of the ambulance's engine noise sweeps up and down with player position on screen
;    N.B. This bug is only observed when the game is run in Kega Fusion 3.64.

; At this point there is no intention of fixing the aforementioned bugs. They
; are charming and I do not have time to do extensive debugging. Perhaps in
; the future I will do a complete review of the program's architecture and
; optimize the code while fixing bugs but this will not happen for a while.

; Acknowledgements and references
; -------------------------------

; 1) Maxim's excellent SMS programming tutorials and example source code at:
;    a) http://www.smspower.org/maxim/HowToProgram/Lesson1AllOnOnePage
;    b) http://www.smspower.org/uploads/Homebrew/Ono-SMS-1.01-src.zip
;    c) http://www.smspower.org/uploads/Development/SN76489-20030421.txt
; 2) Richard Talbot-Watkins' SMS technical documentation at:
;    http://www.smspower.org/uploads/Development/richard.txt
; 3) Charles MacDonald's SMS technical documentation at:
;    http://www.smspower.org/uploads/Development/msvdp-20021112.txt
; 4) Sega Mk III console programming manual at:
;    http://www.smspower.org/Development/OfficialDocumentation?sid=f452272d67f2eedfda78031a91db1dca
; 5) Zilog Z80 programming manual at:
;    http://www.smspower.org/uploads/Development/z80cpu_um.zip

; Licensing
; ---------

; Please feel free to do what you like with this code.

; Insert memory map directives from file
; --------------------------------------

; This includes basic directives to setup the memory configuration of the
; program so that it fits within an SMS ROM. Additionally, this file contains
; all the variable definitions used in the program.

.include "memory_map.asm"

; Insert SDSC tag and SMS rom header
; ----------------------------------
.sdsctag 1.0,"CASEVAC","Combat ambulence simulation","Written by Patrick Beeson 2016"

; Insert boot instructions from file
; ----------------------------------
.include "boot.asm"

; Insert functions from file
; --------------------------
.include "functions.asm"
	
; Main program
; ============
main:                                   ; main function marker
	call IniVDP                         ; initialize the SMS' VDP
	call TurnOnScreen                   ; turn on the screen to display messages/gameplay

	call DrawTitleScreen                ; draw the title screen
	ld bc, 60000                        ; delay execution by bc loop iterations
	call DelayExecution                 ; delay function used to prevent skipping the next screen due controller
	                                    ; signal retention
	call WaitForKeyPress                ; wait until player presses a key on the joypad
	
	call DrawIntroScreen                ; draw the intro screen
	ld bc, 60000                        ; delay execution by bc loop iterations
	call DelayExecution                 ; delay function used to prevent skipping the next screen due controller
	                                    ; signal retention
	call WaitForKeyPress                ; wait until player presses a key on the joypad

	call InitializeVariables            ; set starting parameters for all variables

	MainEventLoop:                      ; top of main event loop (analogous to infinite while loop)
		call SilencePSG                 ; turn off all sounds started during past iteration of the main event loop
		call GetInputs                  ; get input from the player
		call CalcPlayerCurrentColumnRow ; calculate player's current position wrt background image (required to support
		                                ; impassable object collision detection)
		call MovePlayer                 ; update player's x & y coords and map indexes based on user input
		                                ; it is within this function that impassable object collision detection is implemented
		call UpdateTiles                ; update background tile drawings if required
		call DrawLivesAndCas            ; update the lives and casualty score indicator bars
		call CollisionHandler           ; call the function which detects and handles collision events
		call UpdateSprites              ; update sprite drawings
		
		call ResetMapChangeFlag         ; reset MapChangeFlag variable
		
		ld bc, 10                       ; delay execution by bc loop iterations
		call DelayExecution             ; delay function used to slow the game execution to make it more playable
		                                ; titrate the loop iterations prn to make the game playable
		
		jp MainEventLoop                ; return to top of main event loop

; Insert data from file
; ---------------------

; This file consists of all the data used by the game. These include bitmaps for tiles, tilemaps for
; maps and a variety of other misc data.

.include "data.asm"