;==============================================================
; CASEVAC - functions include file
;
; * This file contains the functions called by the main
;   program.
; * This file contains the define directives used throughout
;   the source code
;
; Concept, art and programming by Patrick Beeson
;==============================================================

; Administrative directives
; =========================

; Define constants
; ----------------

; N.B. Not all of these constants are used; some are vestigial components from past
; games and have been left in since they do no harm.

.define VDPControl $bf               ; define VDP control port
.define VDPData $be                  ; define VDP data port
.define SoundControl $7f             ; define the PSG control port
.define VRAMWrite $4000              ; define VRAM start address
.define CRAMWrite $c000              ; define CRAM start address
.define TilemapWrite $38|$40         ; define tile map start address
.define SpritemapWrite $3f|$40       ; define sprite map start address
.define IOPort1 $dc                  ; define input 1 port
.define IOPort2 $dd                  ; define input 2 port
.define P1U %00000001                ; define player 1 up on IOPort1
.define P1D %00000010                ; define player 1 down on IOPort1
.define P1L %00000100                ; define player 1 left on IOPort1
.define P1R %00001000                ; define player 1 right on IOPort1
.define P11 %00010000                ; define player 1 button 1 on IOPort1
.define P12 %00100000                ; define player 1 button 2 on IOPort1
.define P2U %01000000                ; define player 2 up on IOPort1
.define P2D %10000000                ; define player 2 down on IOPort1
.define P2L %00000001                ; define player 2 left on IOPort2
.define P2R %00000010                ; define player 2 right on IOPort2
.define P21 %00000100                ; define player 2 button 1 on IOPort2
.define P22 %00001000                ; define player 2 button 2 on IOPort2
.define ResetButton %00010000        ; define reset button on IOPort2
.define COL %00100000                ; define the sprite collision detect signal from port $bf
.define TopOfMap 0                   ; define y coord of top of map
.define BotOfMap 175                 ; define y coord of bottom of map
.define Startx 128                   ; define player's starting x coord
.define Starty 87                    ; define player's starting y coord
.define Startmapx 1                  ; define player's starting x map index
.define Startmapy 1                  ; define player's starting y map index
.define PlayerUp 1                   ; define player direction up
.define PlayerLeft 2                 ; define player direction left
.define PlayerDown 3                 ; define player direction down
.define PlayerRight 4                ; define player direction right
.define StartLives 10                ; define the starting number of lives the player has
.define StartCas 0                   ; define the starting number of casualties collected/saved
.define MaxCasSaved 10               ; define the maximum number of casualties the player may save
.define ExplosionDuration 20         ; define the explosion duration in iterations of main event loop

; Misc/Administrative functions
; =============================

; A function to initialize variable values
; ----------------------------------------
; Parameters: nil
InitializeVariables:
	; set explosion sprite initial parameters
	ld a, 0
	ld (Explosion.map_x), a   ; set map x
	ld (Explosion.map_y), a   ; set map y
	ld (Explosion.y), a       ; set y coord
	ld (Explosion.x), a       ; set x coord
	ld (Explosion.tile), a    ; set tile 1 index
	ld (Explosion.tile2), a   ; set tile 2 index
	ld bc, 0
	ld (ExplosionCounter), bc ; set explosion counter to zero
	
	; set flag variables
	ld a, $01
	ld (MapChangeFlag), a     ; set MapChangeFlag to 1 which forces UpdateTiles to redraw the background
	ld (WheelsTurningFlag), a ; set WheelsTurningFlag
	ld (EngineSoundFlag), a   ; set EngineSoundFlag
	ld (RPGYMoveFlag), a      ; set RPG Y Move flag

	; set player's initial parameters
	ld a, Startx
	ld (Ambulance.x), a       ; set x coord
	ld a, Starty
	ld (Ambulance.y), a       ; set y coord
	ld a, 0
	ld (Ambulance.tile), a    ; set tile 1 index
	ld a, 0
	ld (Ambulance.tile2), a   ; set tile 2 index
	ld a, Startmapx
	ld (Ambulance.map_x), a   ; set x map index
	ld a, Startmapy
	ld (Ambulance.map_y), a   ; set y map index
	ld bc, StartLives
	ld (Lives), bc            ; set player's starting lives
	ld bc, StartCas
	ld (CasualtiesSaved), bc  ; set player's starting casualties saved score
	ld a, PlayerRight
	ld (DirectionOfTravel), a ; set ambulance's starting direction of travel
	
	; set arty round sprite initial parameters
	ld bc, 10                 ; load number of arty rounds in game to register bc
	ld (NoOfArtyRds), bc      ; set the number of arty rounds in the simulation
	ld hl, ArtyRds            ; load start address of arty round objects into register hl
	ld de, ArtyRdData         ; load start address of arty round data into register de
	
	call INISpriteObjects     ; INI arty object data
	
	; set RPG round sprite initial parameters
	ld bc, 10                 ; load number of RPG rounds in game to register bc
	ld (NoOfRPGRds), bc       ; set the number of RPG rounds in the simulation
	ld hl, RPGRds             ; load start address of RPG round objects into register hl
	ld de, RPGRdData          ; load start address of RPG round data into register de
	
	call INISpriteObjects     ; INI RPG object data
	
	; set IED sprite initial parameters
	ld bc, 10                 ; load number of IEDs in game to register bc
	ld (NoOfIEDs), bc         ; set the number of IEDs in the simulation
	ld hl, IEDs               ; load start address of IED objects into register hl
	ld de, IEDData            ; load start address of IED data into register de
	
	call INISpriteObjects     ; INI IED object data
	
	; set casualty sprite initial parameters
	ld bc, 10                 ; load number of casualties in game to register bc
	ld (NoOfCas), bc          ; set the number of casualties in the simulation
	ld hl, Casualties         ; load start address of casualties objects into register hl
	ld de, CasualtyData       ; load start address of casualty data into register de
	
	call INISpriteObjects     ; INI casualty object data
	
	ret

; A function to initialize multiple sprite object values
; ------------------------------------------------------
; Parameters: bc = number of objects, hl = start address of objects in RAM,
;             de = start address of data to be filled into objects
INISpriteObjects:
	SpriteObjINILoop:          ; top of loop (analogous to for loop)
		ld a, (de)             ; load x map index data into register a
		ld (hl), a             ; send x map index data to address hl
		inc de                 ; increment de to point to next data byte
		inc hl                 ; increment hl to point to object data byte
		ld a, (de)             ; load y map index data into register a
		ld (hl), a             ; send y map index data to address hl
		inc de                 ; increment de to point to next data byte
		inc hl                 ; increment hl to point to next object byte
		ld a, (de)             ; load y coord data into register a
		ld (hl), a             ; send y coord data to address hl
		inc de                 ; increment de to point to next data byte
		inc hl                 ; increment hl to point to next object byte
		ld a, (de)             ; load x coord data into register a
		ld (hl), a             ; send x coord data to address hl
		inc de                 ; increment de to point to next data byte
		inc hl                 ; increment hl to point to next object byte
		ld a, (de)             ; load tile index data into register a
		ld (hl), a             ; send tile index data to address hl
		inc de                 ; increment de to point to next data byte
		inc hl                 ; increment hl to point to next object byte
		ld a, (de)             ; load tile 2 index data into register a
		ld (hl), a             ; send tile 2 index data to address hl
		inc de                 ; increment de to point to next data byte
		inc hl                 ; increment hl to point to next object byte
		dec bc                 ; decrement counter
    	ld a,b                 ; load low byte (b) from counter into register a
		or c                   ; compare a and c; if both are zero then set the nz flag
		jp nz,SpriteObjINILoop ; if nz = 1 then jump to top of loop

	ret

; A function to reset the MapChangeFlag variable
; ----------------------------------------------
; Parameters: nil
ResetMapChangeFlag:
	ld a, $00
	ld (MapChangeFlag), a ; set MapChangeFlag to 0 which prevents the background from being redrawn until a map change occurs	
	ret

; A function to toggle the WheelsTurningFlag variable
; ---------------------------------------------------
; Parameters: nil
ToggleWheelsTurningFlag:
	ld a, (WheelsTurningFlag)
	cp 0                                  ; if WheelsTurningFlag = 0
	jp z, SetTo1                          ; then jump to SetTo1 marker
	cp 1                                  ; if WheelsTurningFlag = 0
	jp z, SetTo0                          ; then jump to SetTo1 marker

	SetTo1:                               ; SetTo1 marker
	ld a, 1
	ld (WheelsTurningFlag), a             ; set WheelsTurningFlag to 1
	jp EndFunctionToggleWheelsTurningFlag ; jump to the end of the function
	
	SetTo0:                               ; SetTo0 marker
	ld a, 0
	ld (WheelsTurningFlag), a             ; set WheelsTurningFlag to 1
	jp EndFunctionToggleWheelsTurningFlag ; jump to the end of the function
	
	EndFunctionToggleWheelsTurningFlag:   ; EndFunctionToggleWheelsTurningFlag marker
	
	ret

; A function to delay execution via a loop
; ----------------------------------------
; Parameters: bc = no of loop interations
DelayExecution:
	DelayLoop:          ; top of loop (analogous to for loop)
		dec bc          ; decrement loop counter
		ld a, b         ; load low byte from bc into register a
		or c            ; compare high and low bytes
		jp nz,DelayLoop ; if result non-zero then jump to top of loop
	
	ret

; A function to end each sprite update loop
; -----------------------------------------
; N.B. This function performs the required pointer advancement, counter decrementing
; and loop end testing operations to complete sprite objects update loops.
; It should also be noted that this function could have been combined with the one below.
; These functions have been kept seperate since the two client modules were designed at
; seperate times using different registers; it is easier to keep both functions than
; to modify one of the two client modules.
; Parameters: de = object pointer, bc = loop iteration counter
EndUpdateLoop:
	inc de ; increment de six times to point to the next sprite object
	inc de
	inc de
	inc de
	inc de
	inc de
	dec bc ; decrement counter
	ld a,b ; load high byte (b) from counter into register a
	or c   ; compare a and c; if both are zero then sets the nz flag
	
	ret
; A function to end each sprite collision handling loop
; -----------------------------------------------------
; N.B. This function performs the required pointer advancement, counter decrementing
; and loop end testing operations to complete sprite collision handling loops.
; It should also be noted that this function could have been combined with the one above.
; These functions have been kept seperate since the two client modules were designed at
; seperate times using different registers; it is easier to keep both functions than
; to modify one of the two client modules.
; Parameters: hl = object pointer, bc = loop iteration counter
EndCollisionLoop:
	inc hl  ; increment hl six times to point to the next sprite object
	inc hl
	inc hl
	inc hl
	inc hl
	inc hl
	dec bc  ; decrement counter
	ld a, b ; load high byte of bc into a
	or c    ; compare c and a
	
	ret

; A function to determine if a sprite has the same x map index as the player for update loops
; -------------------------------------------------------------------------------------------
; It should also be noted that this function could have been combined with the one below.
; These functions have been kept seperate since the two client modules were designed at
; seperate times using different registers; it is easier to keep both functions than
; to modify one of the two client modules.
; Parameters: de = pointer to x map index
CheckXIndexUpdateLoop:
	ld a, (de)              ; load sprite x map index into register a
	ld b, a                 ; shift sprite x map index into register b
	ld a, (Ambulance.map_x) ; load player x map index into register a
	cp b                    ; if sprite and player do not have the same map index then set nz flag
	
	ret

; A function to determine if a sprite has the same x map index as the player for collision handling loops
; -------------------------------------------------------------------------------------------------------
; It should also be noted that this function could have been combined with the one above.
; These functions have been kept seperate since the two client modules were designed at
; seperate times using different registers; it is easier to keep both functions than
; to modify one of the two client modules.
; Parameters: de = pointer to x map index
CheckXIndexCollisionLoop:
	ld a, (hl)              ; load sprite x map index into register a
	ld b, a                 ; shift sprite x map index into register b
	ld a, (Ambulance.map_x) ; load player x map index into register a
	cp b                    ; if sprite and player do not have the same map index then set nz flag
	
	ret
	
; A function to if a sprite has the same y map index as the player for update loops
; ---------------------------------------------------------------------------------
; It should also be noted that this function could have been combined with the one below.
; These functions have been kept seperate since the two client modules were designed at
; seperate times using different registers; it is easier to keep both functions than
; to modify one of the two client modules.
; Parameters: de = pointer to y map index
CheckYIndexUpdateLoop:
	ld a, (de)              ; load sprite y map index into register a
	ld b, a                 ; shift sprite y map index into register b
	ld a, (Ambulance.map_y) ; load player y map index into register a
	cp b                    ; if sprite and player do not have the same map index then set nz flag
	
	ret	

; A function to determine if a sprite has the same y map index as the player for collision handling loops
; -------------------------------------------------------------------------------------------------------
; It should also be noted that this function could have been combined with the one above.
; These functions have been kept seperate since the two client modules were designed at
; seperate times using different registers; it is easier to keep both functions than
; to modify one of the two client modules.
; Parameters: de = pointer to y map index
CheckYIndexCollisionLoop:
	ld a, (hl)              ; load sprite y map index into register a
	ld b, a                 ; shift sprite y map index into register b
	ld a, (Ambulance.map_y) ; load player y map index into register a
	cp b                    ; if sprite and player do not have the same map index then set nz flag
	
	ret
	
; VDP interface functions
; =======================

; A function to set the VDP address to be written to
; --------------------------------------------------
; Parameters: hl = address
SetVDPAddress:
	ld a, l            ; load low byte of address into register a
	out (VDPControl),a ; output low byte of address to VDP control port
	ld a, h            ; load high byte of address into register a
	out (VDPControl),a ; output high byte of address to VDP control port
	ret

; A function to set the VDP registers
; -----------------------------------
; Parameters: VdpData in data include file
SetVDPRegisters:
    ld hl,VdpData           ; load the address at which VdpData starts into register hl
    ld b,VdpDataEnd-VdpData ; load the difference between end and start addressed of VdpData into register b
    ld c,VDPControl         ; load the VDP control port No. ($bf) into register c
    otir                    ; output b bytes of data to port c starting at address hl
	ret

; A Function to clear VRAM
; ------------------------
; Parameters: nil
ClearVRAM:
	; 1. Set VRAM write address to beginning of VRAM
	ld hl,$0000 | VRAMWrite  ; load VRAM write address into register hl
	call SetVDPAddress       ; call function to set VRAM write address
	
    ; 2. Output 16KB of zeroes
    ld bc, $4000             ; counter for 16KB of VRAM
    ClearVRAMLoop:           ; top of loop (analogous to for loop)
        ld a, $00            ; load value to write to VRAM into register a
        out (VDPData), a     ; output contents of register a to port $be (VDP data port)
        dec bc               ; decrement counter
        ld a, b              ; load low byte (b) from counter into register a
        or c                 ; compare a and c; if both are zero then set the nz flag
        jp nz, ClearVRAMLoop ; if counter has not run out then jump to ClearVRAMLoop

	ret

; A function to send data to VDPData
; ----------------------------------
; Parameters: VDP address [see SetVDPAddress], hl = start address of data, bc = end address
;             of data - start address of data
SendToVDPData:
    SendToVDPLoop:          ; top of loop (analogous to for loop)
        ld a,(hl)           ; load data at address stored in hl into register a
        out (VDPData),a     ; output byte in register a to port $be (VDP data port)
        inc hl              ; increment hl so it points to the next data byte
        dec bc              ; decrement counter
        ld a,b              ; load low byte (b) from counter into register a
        or c                ; compare a and c; if both are zero then set the nz flag
        jp nz,SendToVDPLoop ; if nz = 1 then jump to SendToVDPLoop

	ret

; A function to send palette data to VDPData
; ------------------------------------------
; Parameters: PaletteData in data include file
SendPaletteData:
	; 1. Set VRAM write address
	ld hl,$0000 | CRAMWrite         ; load VRAM write address into register hl
	call SetVDPAddress              ; call function to set VRAM write address

    ; 2. Output colour data
    ld hl,PaletteData               ; load the address at which PaletteData starts into register hl
    ld b,PaletteDataEnd-PaletteData ; load the difference between end and start addressed of PaletteData into register b
    ld c,VDPData                    ; load the VDP data port No. ($be) into register c
    otir                            ; output b bytes of data to port c starting at address hl
	ret

; A function to load tiles into VRAM
; ----------------------------------
; Parameters: nil
LoadTiles:
	; 1. Set VRAM write address to beginning of VRAM
	ld hl,$0000 | VRAMWrite     ; load VRAM write address into register hl
	call SetVDPAddress          ; call function to set VRAM write address
	
    ; 2. Output tile data
	; N.B. a loop must be used here since otir only works for data that is 256 bytes or less
    ld hl,TileData              ; load the address at which TileData starts into register hl
    ld bc,TileDataEnd-TileData  ; load the difference between end and start addresses of TileData
                                ; into register bc. This will be used as a counter
	call SendToVDPData          ; call function to send data to VDP Data port
	ret
	
; A function to initialize the VDP
; --------------------------------
; Parameters: nil
IniVDP:
	call SetVDPRegisters ; set VDP registers to initialize VDP
	call ClearVRAM       ; clear VRAM
	call SendPaletteData ; send palette data to the VDP
	call LoadTiles       ; send tile data to VDP
	ret

; A function to turn on the screen
; --------------------------------
; Parameters: nil
TurnOnScreen:
    ld a,%01000000
;          |||| |`- Zoomed sprites -> 16x16 pixels
;          |||| `-- Doubled sprites -> 2 tiles per sprite, 8x16
;          |||`---- 30 row/240 line mode
;          ||`----- 28 row/224 line mode
;          |`------ VBlank interrupts
;          `------- Enable display
    out (VDPControl),a ; send screen activation byte to VDP control port
    ld a,$81           ; load $81 into register a
    out (VDPControl),a ; send register a to VDP control port
	ret

; A function to turn off the screen
; ---------------------------------
; Parameters: nil
TurnOffScreen:
    ld a,%00000000
;          |||| |`- Zoomed sprites -> 16x16 pixels
;          |||| `-- Doubled sprites -> 2 tiles per sprite, 8x16
;          |||`---- 30 row/240 line mode
;          ||`----- 28 row/224 line mode
;          |`------ VBlank interrupts
;          `------- Enable display
    out (VDPControl),a ; send screen deactivation byte to VDP control port
    ld a,$81           ; load $81 into register a
    out (VDPControl),a ; send register a to VDP control port
	ret

; PSG interface functions
; =======================

; A function to slience the PSG
; -----------------------------
; Parameters: nil
SilencePSG:
	ld hl, ClearPSGData                  ; load start address of PSG silencing table into register hl
	ld c, SoundControl                   ; load sound control port into register c
	ld b, ClearPSGDataEnd - ClearPSGData ; load table size into register b
	otir                                 ; output data to the PSG
	ret

; A function to turn on track movement sound
; ------------------------------------------	
; Parameters: nil
TrackMvmtPSG:
	ld a, (EngineSoundFlag)
	cp 1                                             ; if EngineSoundFlag = 1 then
	jp z, PlayEngineSound                            ; jump to PlayEngineSound marker
	cp 0                                             ; if EngineSoundFlag = 0 then
	jp z, DoNotPlayEngineSound                       ; jump to DoNotPlayEngineSound
	
	PlayEngineSound:                                 ; PlayEngineSound marker
		ld hl, TrackMvmtPSGData                      ; load start address of track movement data table into register hl
		ld c, SoundControl                           ; load sound control port into register c
		ld b, TrackMvmtPSGDataEnd - TrackMvmtPSGData ; load table size into register b
		otir                                         ; output data to the PSG
		
		ld a, 0
		ld (EngineSoundFlag), a                      ; set EngineSoundFlag to 0
		
		jp EndFunctionTrackMvmtPSG                   ; jump to end of function
	
	DoNotPlayEngineSound:                            ; DoNotPlayEngineSound marker
		ld a, 1
		ld (EngineSoundFlag), a                      ; set EngineSoundFlag to 1
		
		jp EndFunctionTrackMvmtPSG                   ; jump to end of function
	
	EndFunctionTrackMvmtPSG:                         ; EndFunctionTrackMvmtPSG marker
	
	ret
	
; Background game graphics functions	
; ==================================

; It should be noted that there are seperate functions defined to draw each map.
; This may seem inefficient but this design feature is intentional. These functions
; are all called from the update tiles function; if these functions had not have been
; seperately defined, their code would have been placed in the update tiles
; function. This would have made little difference in overall code length but would
; have added to the complexity of the update tiles function.

; A function to draw the map 0,0 screen
; -------------------------------------
; Parameters: nil
DrawMap0_0Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map0_0           ; load the address at which screen data starts into register hl
    ld bc,Map0_0End-Map0_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port
	
	; 3. Output tilemap data to BGMatrix
	ld hl,Map0_0           ; load the address at which screen data starts into register hl
    ld bc,Map0_0End-Map0_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 1,0 screen
; -------------------------------------
; Parameters: nil
DrawMap1_0Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map1_0           ; load the address at which screen data starts into register hl
    ld bc,Map1_0End-Map1_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map1_0           ; load the address at which screen data starts into register hl
    ld bc,Map1_0End-Map1_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 2,0 screen
; -------------------------------------
; Parameters: nil
DrawMap2_0Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map2_0           ; load the address at which screen data starts into register hl
    ld bc,Map2_0End-Map2_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map2_0           ; load the address at which screen data starts into register hl
    ld bc,Map2_0End-Map2_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 3,0 screen
; -------------------------------------
; Parameters: nil
DrawMap3_0Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map3_0           ; load the address at which screen data starts into register hl
    ld bc,Map3_0End-Map3_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map3_0           ; load the address at which screen data starts into register hl
    ld bc,Map3_0End-Map3_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 4,0 screen
; -------------------------------------
; Parameters: nil
DrawMap4_0Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map4_0           ; load the address at which screen data starts into register hl
    ld bc,Map4_0End-Map4_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map4_0           ; load the address at which screen data starts into register hl
    ld bc,Map4_0End-Map4_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution	

; A function to draw the map 5,0 screen
; -------------------------------------
; Parameters: nil
DrawMap5_0Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map5_0           ; load the address at which screen data starts into register hl
    ld bc,Map5_0End-Map5_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map5_0           ; load the address at which screen data starts into register hl
    ld bc,Map5_0End-Map5_0 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 0,1 screen
; -------------------------------------
; Parameters: nil
DrawMap0_1Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map0_1           ; load the address at which screen data starts into register hl
    ld bc,Map0_1End-Map0_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map0_1           ; load the address at which screen data starts into register hl
    ld bc,Map0_1End-Map0_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix	

	ret                    ; return to last execution

; A function to draw the map 1,1 screen
; -------------------------------------
; Parameters: nil
DrawMap1_1Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map1_1           ; load the address at which screen data starts into register hl
    ld bc,Map1_1End-Map1_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map1_1           ; load the address at which screen data starts into register hl
    ld bc,Map1_1End-Map1_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 2,1 screen
; -------------------------------------
; Parameters: nil
DrawMap2_1Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map2_1           ; load the address at which screen data starts into register hl
    ld bc,Map2_1End-Map2_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map2_1           ; load the address at which screen data starts into register hl
    ld bc,Map2_1End-Map2_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 3,1 screen
; -------------------------------------
; Parameters: nil
DrawMap3_1Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map3_1           ; load the address at which screen data starts into register hl
    ld bc,Map3_1End-Map3_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map3_1           ; load the address at which screen data starts into register hl
    ld bc,Map3_1End-Map3_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 4,1 screen
; -------------------------------------
; Parameters: nil
DrawMap4_1Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map4_1           ; load the address at which screen data starts into register hl
    ld bc,Map4_1End-Map4_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map4_1           ; load the address at which screen data starts into register hl
    ld bc,Map4_1End-Map4_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution	

; A function to draw the map 5,1 screen
; -------------------------------------
; Parameters: nil
DrawMap5_1Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map5_1           ; load the address at which screen data starts into register hl
    ld bc,Map5_1End-Map5_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map5_1           ; load the address at which screen data starts into register hl
    ld bc,Map5_1End-Map5_1 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 0,2 screen
; -------------------------------------
; Parameters: nil
DrawMap0_2Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map0_2           ; load the address at which screen data starts into register hl
    ld bc,Map0_2End-Map0_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map0_2           ; load the address at which screen data starts into register hl
    ld bc,Map0_2End-Map0_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 1,2 screen
; -------------------------------------
; Parameters: nil
DrawMap1_2Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map1_2           ; load the address at which screen data starts into register hl
    ld bc,Map1_2End-Map1_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map1_2           ; load the address at which screen data starts into register hl
    ld bc,Map1_2End-Map1_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 2,2 screen
; -------------------------------------
; Parameters: nil
DrawMap2_2Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map2_2           ; load the address at which screen data starts into register hl
    ld bc,Map2_2End-Map2_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map2_2           ; load the address at which screen data starts into register hl
    ld bc,Map2_2End-Map2_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 3,2 screen
; -------------------------------------
; Parameters: nil
DrawMap3_2Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map3_2           ; load the address at which screen data starts into register hl
    ld bc,Map3_2End-Map3_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map3_2           ; load the address at which screen data starts into register hl
    ld bc,Map3_2End-Map3_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to draw the map 4,2 screen
; -------------------------------------
; Parameters: nil
DrawMap4_2Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map4_2           ; load the address at which screen data starts into register hl
    ld bc,Map4_2End-Map4_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map4_2           ; load the address at which screen data starts into register hl
    ld bc,Map4_2End-Map4_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution	

; A function to draw the map 5,2 screen
; -------------------------------------
; Parameters: nil
DrawMap5_2Screen:          ; function definition
	; 1. Set VRAM write address to tile map location
    ld a,$00               ; load $00 into register a
	out (VDPControl),a     ; output register a to VDP control port
	ld a,TilemapWrite      ; load tile map location into register a
	out (VDPControl),a     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,Map5_2           ; load the address at which screen data starts into register hl
    ld bc,Map5_2End-Map5_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToVDPData     ; call function to send data to VDP Data port

	; 3. Output tilemap data to BGMatrix
	ld hl,Map5_2           ; load the address at which screen data starts into register hl
    ld bc,Map5_2End-Map5_2 ; load the difference between end and start addresses of screen data
	                       ; into register bc. This will be used as a counter
	call SendToBGMatrix    ; call function to send data to BGMatrix
	
	ret                    ; return to last execution

; A function to update tiles
; --------------------------
; Parameters: nil
UpdateTiles:
	ld a, (MapChangeFlag)
	cp $00                        ; if the map has not changed then
	jp z, EndFunctionUpdateTiles  ; jump to the end of the function without doing anything
	
	ld a, (Ambulance.map_y)       ; load y map index
	cp $00                        ; if y index = 0 then
	jp z, Row0                    ; jump to Row0
	cp $01                        ; if y index = 1 then
	jp z, Row1                    ; jump to Row1
	cp $02                        ; if y index = 2 then
	jp z, Row2                    ; jump to Row2
	
	; row 0 if tree
	Row0:                         ; Row0 marker
		ld a, (Ambulance.map_x)   ; load x map index
		cp $00                    ; if x index = 0 then
		jp z, DMap0_0             ; jump to DMap0_0
		cp $01                    ; if x index = 1 then
		jp z, DMap1_0             ; jump to DMap1_0
		cp $02                    ; if x index = 2 then
		jp z, DMap2_0             ; jump to DMap2_0
		cp $03                    ; if x index = 3 then
		jp z, DMap3_0             ; jump to DMap3_0
		cp $04                    ; if x index = 4 then
		jp z, DMap4_0             ; jump to DMap4_0
		cp $05                    ; if x index = 5 then
		jp z, DMap5_0             ; jump to DMap5_0
		
	; row 1 if tree
	Row1:
		ld a, (Ambulance.map_x)   ; load x map index
		cp $00                    ; if x index = 0 then
		jp z, DMap0_1             ; jump to DMap0_1
		cp $01                    ; if x index = 1 then
		jp z, DMap1_1             ; jump to DMap1_1
		cp $02                    ; if x index = 2 then
		jp z, DMap2_1             ; jump to DMap2_1
		cp $03                    ; if x index = 0 then
		jp z, DMap3_1             ; jump to DMap3_1
		cp $04                    ; if x index = 4 then
		jp z, DMap4_1             ; jump to DMap4_1
		cp $05                    ; if x index = 5 then
		jp z, DMap5_1             ; jump to DMap5_1
	
	; row 2 if tree
	Row2:
		ld a, (Ambulance.map_x)   ; load x map index
		cp $00                    ; if x index = 0 then
		jp z, DMap0_2             ; jump to DMap0_2
		cp $01                    ; if x index = 1 then
		jp z, DMap1_2             ; jump to DMap1_2
		cp $02                    ; if x index = 2 then
		jp z, DMap2_2             ; jump to DMap2_2
		cp $03                    ; if x index = 0 then
		jp z, DMap3_2             ; jump to DMap3_2
		cp $04                    ; if x index = 4 then
		jp z, DMap4_2             ; jump to DMap4_2
		cp $05                    ; if x index = 5 then
		jp z, DMap5_2             ; jump to DMap5_2
	
	jp EndFunctionUpdateTiles     ; if none of the conditions are met jump to end of function
	
	; Row 0
	DMap0_0:                      ; DMap0_0 marker
		call DrawMap0_0Screen     ; draw map 0_0 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap1_0:                      ; DMap1_0 marker
		call DrawMap1_0Screen     ; draw map 1_0 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	DMap2_0:                      ; DMap2_0 marker
		call DrawMap2_0Screen     ; draw map 2_0 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap3_0:                      ; DMap3_0 marker
		call DrawMap3_0Screen     ; draw map 3_0 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	DMap4_0:                      ; DMap4_0 marker
		call DrawMap4_0Screen     ; draw map 4_0 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap5_0:                      ; DMap5_0 marker
		call DrawMap5_0Screen     ; draw map 5_0 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	; Row 1
	DMap0_1:                      ; DMap0_1 marker
		call DrawMap0_1Screen     ; draw map 0_1 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap1_1:                      ; DMap1_1 marker
		call DrawMap1_1Screen     ; draw map 1_1 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	DMap2_1:                      ; DMap2_1 marker
		call DrawMap2_1Screen     ; draw map 2_1 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap3_1:                      ; DMap3_1 marker
		call DrawMap3_1Screen     ; draw map 3_1 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	DMap4_1:                      ; DMap4_1 marker
		call DrawMap4_1Screen     ; draw map 4_1 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap5_1:                      ; DMap5_1 marker
		call DrawMap5_1Screen     ; draw map 5_1 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
		
	; Row 2
	DMap0_2:                      ; DMap0_2 marker
		call DrawMap0_2Screen     ; draw map 0_2 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap1_2:                      ; DMap1_2 marker
		call DrawMap1_2Screen     ; draw map 1_2 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	DMap2_2:                      ; DMap2_2 marker
		call DrawMap2_2Screen     ; draw map 2_2 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap3_2:                      ; DMap3_2 marker
		call DrawMap3_2Screen     ; draw map 3_2 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	DMap4_2:                      ; DMap4_2 marker
		call DrawMap4_2Screen     ; draw map 4_2 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function
	
	DMap5_2:                      ; DMap5_2 marker
		call DrawMap5_2Screen     ; draw map 5_2 on the screen
		jp EndFunctionUpdateTiles ; jump to End of function

	EndFunctionUpdateTiles:       ; EndFunctionUpdateTiles marker
	
	ret

; A function to draw players lives and casualty score indicators
; --------------------------------------------------------------
; Parameters: nil
DrawLivesAndCas:
	; 1. Set VRAM write address to tile map location + offset to put lives and cas at the bottom of the screen
    ld a,$c0                         ; load $c0 into register a (low byte) - this is the offset to place indicators at the bottom
	out (VDPControl),a               ; output register a to VDP control port
	ld a,$3d                         ; load tile map location into register a (high byte)
	out (VDPControl),a               ; output register a to VDP control port
	
	; 2. Send lives and cas data to VDP
	; check to see if player has any lives to draw
	ld bc, (Lives)                   ; load players lives into register bc
	ld a, c                          ; load low byte of bc into register a
	cp $00                           ; if player has no lives then
	jp z, InterveningBlanks          ; jump to the intervening blanks loop
	
	; Send lives
	ld bc, (Lives)                   ; set a counter to equal number of player lives
	HeartsLoop:                      ; top of loop (analogous to for loop)
        ld a,$00                     ; load low end of heart index into register a
        out (VDPData),a              ; output byte in register a to port $be (VDP data port)
        ld a,$52	                 ; load high end of heart index into register a
		out (VDPData),a              ; output byte in register a to port $be (VDP data port)
        dec bc                       ; decrement counter
        ld a,b                       ; load low byte (b) from counter into register a
        or c                         ; compare a and c; if both are zero then sets the nz flag
        jp nz,HeartsLoop             ; if nz = 1 then jump to HeartsLoop
	
	InterveningBlanks:               ; InterveningBlanks marker
	
	; Send intervening blanks
	ld bc, (Lives)                   ; load the number of players lives into register a
	ld a, 15                         ; load the tile coord of the mid point of the screen into register a
	sub c                            ; subtract register c from a and store result in a
	ld c, a                          ; load register a into register c (basically, this sets counter bc equal to the difference between 15 and player lives)
	BlanksLoop:                      ; top of loop (analogous to for loop)
		ld a,$00                     ; load blank tile index
        out (VDPData),a              ; output byte in register a to port $be (VDP data port)
		out (VDPData),a              ; output byte in register a to port $be (VDP data port)
		dec bc                       ; decrement counter
        ld a,b                       ; load low byte (b) from counter into register a
        or c                         ; compare a and c; if both are zero then sets the nz flag
        jp nz,BlanksLoop             ; if nz = 1 then jump to BlanksLoop
	
	; check to see if player has any casualties saved to draw
	ld bc, (CasualtiesSaved)         ; load casualty score into register bc
	ld a, c                          ; load low byte of bc into register a
	cp $00                           ; if player has no cas saved then
	jp z, EndFunctionDrawLivesAndCas ; jump to the end of the function
	
	; Send casualties saved
	ld bc, (CasualtiesSaved)         ; set a counter to equal number of cas saved
	CasSavedLoop:                    ; top of loop (analogous to for loop)
        ld a,$00                     ; load low end of cas saved index into register a
        out (VDPData),a              ; output byte in register a to port $be (VDP data port)
        ld a,$53	                 ; load high end of cas saved index into register a
		out (VDPData),a              ; output byte in register a to port $be (VDP data port)
        dec bc                       ; decrement counter
        ld a,b                       ; load low byte (b) from counter into register a
        or c                         ; compare a and c; if both are zero then sets the nz flag
        jp nz,CasSavedLoop           ; if nz = 1 then jump to CasSavedLoop
	
	EndFunctionDrawLivesAndCas:      ; EndFunctionDrawLivesAndCas marker
	
	ret

; A function to draw the title screen
; -----------------------------------
; Parameters: nil
DrawTitleScreen: 
	; 1. Set VRAM write address to tile map location
    ld a,$00                         ; load $00 into register a
	out (VDPControl),a               ; output register a to VDP control port
	ld a,TilemapWrite                ; load tile map location into register a
	out (VDPControl),a               ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,TitleScreen                ; load the address at which screen data starts into register hl
    ld bc,TitleScreenEnd-TitleScreen ; load the difference between end and start addresses of screen data
	                                 ; into register bc. This will be used as a counter
	call SendToVDPData               ; call function to send data to VDP Data port
	ret

; A function to draw the intro screen
; -----------------------------------
; Parameters: nil
DrawIntroScreen:
	; 1. Set VRAM write address to tile map location
    ld a,$00                         ; load $00 into register a
	out (VDPControl),a               ; output register a to VDP control port
	ld a,TilemapWrite                ; load tile map location into register a
	out (VDPControl),a               ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,IntroScreen                ; load the address at which screen data starts into register hl
    ld bc,IntroScreenEnd-IntroScreen ; load the difference between end and start addresses of screen data
	                                 ; into register bc. This will be used as a counter
	call SendToVDPData               ; call function to send data to VDP Data port
	ret

; A function to draw the game over screen
; ---------------------------------------
; Parameters: nil
DrawGameOverScreen:
	call SilencePSG                        ; silence the PSG
	
	; 1. Set VRAM write address to tile map location
    ld a,$00                               ; load $00 into register a
	out (VDPControl),a                     ; output register a to VDP control port
	ld a,TilemapWrite                      ; load tile map location into register a
	out (VDPControl),a                     ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,GameOverScreen                   ; load the address at which screen data starts into register hl
    ld bc,GameOverScreenEnd-GameOverScreen ; load the difference between end and start addresses of screen data
	                                       ; into register bc. This will be used as a counter
	call SendToVDPData                     ; call function to send data to VDP Data port

	; 3. Wait for user input
	ld bc, 60000                           ; delay execution by bc loop iterations
	call DelayExecution                    ; delay function used to prevent skipping the next screen due controller
	                                       ; signal retention
	call WaitForKeyPress                   ; wait for user to press anykey
		
	call TurnOffScreen                     ; turn off the screen so that it doesn't look all messed up when the game restarts
	
	jp main                                ; jump to the begining of the game
	
	ret

; A function to draw the win screen
; ---------------------------------
; Parameters: nil
DrawWinScreen:
	call SilencePSG              ; silence the PSG
	
	; 1. Set VRAM write address to tile map location
    ld a,$00                     ; load $00 into register a
	out (VDPControl),a           ; output register a to VDP control port
	ld a,TilemapWrite            ; load tile map location into register a
	out (VDPControl),a           ; output register a to VDP control port
	
    ; 2. Output tilemap data
    ld hl,WinScreen              ; load the address at which screen data starts into register hl
    ld bc,WinScreenEnd-WinScreen ; load the difference between end and start addresses of screen data
	                             ; into register bc. This will be used as a counter
	call SendToVDPData           ; call function to send data to VDP Data port

	; 3. Wait for user input
	ld bc, 60000                 ; delay execution by bc loop iterations
	call DelayExecution          ; delay function used to prevent skipping the next screen due controller
	                             ; signal retention
	call WaitForKeyPress         ; wait for user to press anykey
		
	call TurnOffScreen           ; turn off the screen so that it doesn't look all messed up when the game restarts
	
	jp main                      ; jump to the begining of the game
	
	ret	
	
; Foreground game graphics functions
; ==================================
	
; A function to update sprites
; ----------------------------
; Parameters: nil
UpdateSprites:
	; update explosion sprite
	ld a, (Explosion.y)
	ld (SpriteTable), a                ; send explosion y coord to local sprite table
	ld a, (Explosion.x)
	ld (SpriteTable + 128), a          ; send explosion x coord to local sprite table
	ld a, (Explosion.tile)
	ld (SpriteTable + 129), a          ; send explosion tile index to local sprite table
	
	ld bc, (ExplosionCounter)          ; load the value of explosion counter into register bc
	dec bc                             ; decrement bc
	ld (ExplosionCounter), bc          ; send decremented explosion counter back to ram
	ld a,b                             ; load high byte of bc into a
	or c                               ; compare a and c
	jp nz, ExplosionOnScreen           ; if counter has not expired, jump to ExplosionOnScreen marker

	ld a, 0
	ld (Explosion.tile), a             ; set explosion sprite tile index to 0 to blank it
	
	ExplosionOnScreen:                 ; explosion on screen marker
	
	; update ambulance sprites
	ld a, (DirectionOfTravel)          ; load direction of travel into register a
	
	; if tree to call the correct routines to draw the ambulance pointing in the correct direction
	cp PlayerUp                        ; if the player is facing up then
	jp z, AmbPointingUp                ; jump to the code that points the ambulance up
	
	cp PlayerDown                      ; if the player is facing down then
	jp z, AmbPointingDown              ; jump to the code that points the ambulance down
	
	cp PlayerLeft                      ; if the player is facing left then
	jp z, AmbPointingLeft              ; jump to the code that points the ambulance left
	
	cp PlayerRight                     ; if the player is facing right then
	jp z, AmbPointingRight             ; jump to the code that points the ambulance right
	
	jp SetupSpriteTablePointers        ; if none of the above conditions are met then jump
	                                   ; to the next part of the sprite update algorithm
	
	; routines to draw the ambulance pointing in the correct direction
	AmbPointingUp:                     ; AmbPointingUp marker
		ld a, (Ambulance.y)            ; load ambulance's fwd y position into register a
		ld (SpriteTable + 1), a        ; send ambulance's fwd y position to the local sprite table
		add a, 8                       ; add 8 to register a
		ld (SpriteTable + 2), a        ; send ambulance's rear y position to the local sprite table
		ld a, (Ambulance.x)            ; load ambulance's fwd x position into register a
		ld (SpriteTable + 130), a      ; send ambulance's fwd x position to the local sprite table
		ld (SpriteTable + 132), a      ; send ambulance's rear x position to the local sprite table
		
		ld a, (WheelsTurningFlag)      ; load WheelsTurningFlag into register a
		cp 0                           ; if WheelsTurningFlag = 0 then
		jp z, UpWheelState0            ; jump to UpWheelState0 marker
		cp 1                           ; if WheelsTurningFlag = 1 then
		jp z, UpWheelState1            ; jump to UpWheelState1 marker
		
		UpWheelState0:                 ; UpWheelState0 marker
			ld a, 41                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 40                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingUp            ; jump to EndAmbPointingUp marker
	
		UpWheelState1:                 ; UpWheelState1 marker
			ld a, 43                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 42                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingUp            ; jump to EndAmbPointingUp marker
	
		EndAmbPointingUp:              ; EndAmbPointingUp marker
	jp SetupSpriteTablePointers        ; jump to next part of sprite update algorithm
	
	AmbPointingDown:                   ; AmbPointingDown marker
		ld a, (Ambulance.y)            ; load ambulance's fwd y position into register a
		ld (SpriteTable + 1), a        ; send ambulance's fwd y position to the local sprite table
		sub 8                          ; subtract 8 from register a
		ld (SpriteTable + 2), a        ; send ambulance's rear y position to the local sprite table
		ld a, (Ambulance.x)            ; load ambulance's fwd x position into register a
		ld (SpriteTable + 130), a      ; send ambulance's fwd x position to the local sprite table
		ld (SpriteTable + 132), a      ; send ambulance's rear x position to the local sprite table
		
		ld a, (WheelsTurningFlag)      ; load WheelsTurningFlag into register a
		cp 0                           ; if WheelsTurningFlag = 0 then
		jp z, DownWheelState0          ; jump to DownWheelState0 marker
		cp 1                           ; if WheelsTurningFlag = 1 then
		jp z, DownWheelState1          ; jump to DownWheelState1 marker
		
		DownWheelState0:               ; DownWheelState0 marker
			ld a, 49                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 48                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingDown          ; jump to EndAmbPointingDown marker
	
		DownWheelState1:               ; DownWheelState1 marker
			ld a, 51                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 50                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingDown          ; jump to EndAmbPointingDown marker
	
		EndAmbPointingDown:            ; EndAmbPointingUp marker
	jp SetupSpriteTablePointers        ; jump to next part of sprite update algorithm
	
	AmbPointingLeft:                   ; AmbPointingLeft marker
		ld a, (Ambulance.y)            ; load ambulance's fwd y position into register a
		ld (SpriteTable + 1), a        ; send ambulance's fwd y position to the local sprite table
		ld (SpriteTable + 2), a        ; send ambulance's rear y position to the local sprite table
		ld a, (Ambulance.x)            ; load ambulance's fwd x position into register a
		ld (SpriteTable + 130), a      ; send ambulance's fwd x position to the local sprite table
		add a, 8                       ; add 8 to register a
		ld (SpriteTable + 132), a      ; send ambulance's rear x position to the local sprite table
		
		ld a, (WheelsTurningFlag)      ; load WheelsTurningFlag into register a
		cp 0                           ; if WheelsTurningFlag = 0 then
		jp z, LeftWheelState0          ; jump to LeftWheelState0 marker
		cp 1                           ; if WheelsTurningFlag = 1 then
		jp z, LeftWheelState1          ; jump to LeftWheelState1 marker
		
		LeftWheelState0:               ; LeftWheelState0 marker
			ld a, 44                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 45                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingLeft          ; jump to EndAmbPointingLeft marker
	
		LeftWheelState1:               ; LeftWheelState1 marker
			ld a, 46                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 47                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingLeft          ; jump to EndAmbPointingLeft marker
	
		EndAmbPointingLeft:            ; EndAmbPointingLeft marker
	jp SetupSpriteTablePointers        ; jump to next part of sprite update algorithm
	
	AmbPointingRight:                  ; AmbPointingRight marker
		ld a, (Ambulance.y)            ; load ambulance's fwd y position into register a
		ld (SpriteTable + 1), a        ; send ambulance's fwd y position to the local sprite table
		ld (SpriteTable + 2), a        ; send ambulance's rear y position to the local sprite table
		ld a, (Ambulance.x)            ; load ambulance's fwd x position into register a
		ld (SpriteTable + 130), a      ; send ambulance's fwd x position to the local sprite table
		sub 8                          ; subtract 8 from register a
		ld (SpriteTable + 132), a      ; send ambulance's rear x position to the local sprite table
		
		ld a, (WheelsTurningFlag)      ; load WheelsTurningFlag into register a
		cp 0                           ; if WheelsTurningFlag = 0 then
		jp z, RightWheelState0         ; jump to RightWheelState0 marker
		cp 1                           ; if WheelsTurningFlag = 1 then
		jp z, RightWheelState1         ; jump to RightWheelState1 marker
		
		RightWheelState0:              ; RightWheelState0 marker
			ld a, 37                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 36                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingRight         ; jump to EndAmbPointingRight marker
	
		RightWheelState1:              ; RightWheelState1 marker
			ld a, 39                   ; load appropriate tile index into register a
			ld (SpriteTable + 131), a  ; send appropriate tile index to local sprite table
			ld a, 38                   ; load appropriate tile index into register a
			ld (SpriteTable + 133), a  ; send appropriate tile index to local sprite table
		jp EndAmbPointingRight         ; jump to EndAmbPointingRight marker
	
		EndAmbPointingRight:           ; EndAmbPointingRight marker
	jp SetupSpriteTablePointers        ; jump to next part of sprite update algorithm

	SetupSpriteTablePointers:          ; SetupSpriteTablePointers marker
	
	; setup pointers for sprite table updates
	ld hl, SpriteTable + 3             ; load start address in local sprite table where mob data is to be found
	ld (SpriteTableYPointer), hl       ; load start address in local sprite table where mob data is to be found
	                                   ; into a pointer
	
	ld hl, SpriteTable + 134           ; load start address in local sprite table where mob data is to be found
	ld (SpriteTableXTPointer), hl      ; load start address in local sprite table where mob data is to be found
	                                   ; into a pointer
	
	; if map hasn't changed then jump to mob update part of the function
	ld a, (MapChangeFlag)              ; load map change status flag into register a
	cp $00                             ; if the map has not changed then
	jp z, MobSpriteUpdate              ; jump to the mob update loops
	
	; clear out sprite table
	ld bc, 256                         ; load size of sprite table into counter
	ld hl, SpriteTable                 ; load address of local sprite table
	SpriteTableClearingLoop:           ; top of loop (analogous to for loop)
		ld (hl), $00                   ; zeroize sprite table byte at (hl)
		inc hl                         ; increment sprite table pointer
		dec bc                         ; decrement counter
		ld a, b                        ; load high byte of bc into a
		or c                           ; compare high and low bytes; if non-zero
		jp nz, SpriteTableClearingLoop ; jump to top of loop

	; send zeroized sprite map to the VDP
    ld a,$00                           ; load $00 into register a
	out (VDPControl),a                 ; output register a to VDP control port
	ld a,SpritemapWrite                ; load sprite map location into register a
	out (VDPControl),a                 ; output register a to VDP control port
	ld hl,SpriteTable                  ; load address of local RAM sprite table into register hl
	ld bc,256                          ; load the size of the local RAM sprite table in bytes to register bc
	call SendToVDPData                 ; send the local RAM sprite table to the VDP sprite table in VRAM
		
	MobSpriteUpdate:                   ; MobSpriteUpdate marker
		
	; update arty rounds sprites
    ld bc, (NoOfArtyRds)               ; load number of arty rounds into counter register bc
	ld de, ArtyRds				       ; load start address of arty round objects into register de
	ld hl, (SpriteTableYPointer)       ; load local sprite table address to write y coord to into register hl
	
	ArtyYUpdateLoop:                   ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndArtyYUpdateLoop      ; jump to the end of the arty round Y Update loop

		inc de                         ; increment de to point to arty round y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndArtyYUpdateLoop      ; jump to the end of the arty round Y Update loop
		
		inc de                         ; increment de to point at arty round y coordinate value
		
		; move arty round down
		ld a, (de)                     ; load arty round y coord into register a
		inc a                          ; decrement register a (i.e. move arty round down)
		ld (de), a                     ; set new arty round y coord
		
		; write y coord for bottom half of arty round to sprite table
		ld a, (de)                     ; load arty round y coordinate into register a
		ld (hl), a                     ; write arty round y coord to local sprite table
		
		inc hl                         ; increment hl to point to y coord for top half of arty round in local sprite table
		
		; write y coord for top half of arty round to sprite table
		sub 8                          ; subtract 8 from register a
		ld (hl), a                     ; write arty round y coord to local sprite table
		
		inc hl                         ; increment hl to point to next arty round y coord location in local sprite table
		
		EndArtyYUpdateLoop:            ; EndArtyYUpdateLoop marker
		    pop de                     ; retrieve start of loop de pointer and then increment to next arty round object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,ArtyYUpdateLoop      ; if nz = 1 then jump to ArtyYUpdateLoop
	
	ld (SpriteTableYPointer), hl       ; update sprite table Y pointer
	
	ld bc, (NoOfArtyRds)               ; load number of arty rounds into counter register bc
	ld de, ArtyRds				       ; load start address of arty round objects into register de
	ld hl, (SpriteTableXTPointer)      ; load local sprite table address to write x coord and tile indexes to into register hl
	
	ArtyXTUpdateLoop:                  ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndArtyXTUpdateLoop     ; jump to the end of the arty rounds XT Update loop
		
		inc de                         ; increment de to point to arty rounds y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndArtyXTUpdateLoop     ; jump to the end of the arty rounds XT Update loop
		
		inc de                         ; increment de twice to point to arty rounds x coord
		inc de
		
		; set x coord of bottom half of arty round
		ld a, (de)                     ; load arty round x coordinate into register a
		ld (hl), a                     ; write arty round x coord to local sprite table
		
		; set x coord of top half of arty round
		inc hl                         ; increment hl twice to point to x coord of top half of arty round in local sprite table
		inc hl
		ld (hl), a                     ; write arty round x coord to local sprite table
		
		; set tile index of bottom half of arty round
		dec hl                         ; decrement hl to point to arty round tile index location in local sprite table
		inc de                         ; increment de to point to the arty round tile index bottom
		ld a, (de)                     ; load arty round tile index into register a
		ld (hl), a                     ; write arty round tile index to local sprite table
		
		; set tile index of top half of arty round
		inc hl                         ; increment hl twice to point to top half of arty round tile index in local sprite table
		inc hl
		inc de                         ; increment de to point to the arty round tile index top
		ld a, (de)                     ; load arty round tile index into register a
		ld (hl), a                     ; write arty round tile index to local sprite table
		
		inc hl                         ; increment hl to point to next arty x coord location in local sprite table
		
		EndArtyXTUpdateLoop:
			pop de                     ; retrieve start of loop de pointer and then increment to next object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,ArtyXTUpdateLoop     ; if nz = 1 then jump to ArtyXTUpdateLoop
	
	ld (SpriteTableXTPointer), hl      ; update sprite table XT pointer
	
	; update RPG rounds sprites
    ld bc, (NoOfRPGRds)                ; load number of RPG rounds into counter register bc
	ld de, RPGRds				       ; load start address of RPG round objects into register de
	ld hl, (SpriteTableYPointer)       ; load local sprite table address to write y coord to into register hl
	
	RPGYUpdateLoop:                    ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndRPGYUpdateLoop       ; jump to the end of the RPG round Y Update loop

		inc de                         ; increment de to point to RPG round y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndRPGYUpdateLoop       ; jump to the end of the RPG round Y Update loop
		
		inc de                         ; increment de to point at arty round y coordinate value
		
		; if RPG Y Move Flag is 0 then set to 1 then jump to end of function (this slows RPG homing down by 1/2)
		ld a, (RPGYMoveFlag)           ; load RPG Y move flag into register a
		cp $00                         ; compare with zero
		ld a, $01                      ; load one into register a
		ld (RPGYMoveFlag), a           ; set RPG Y move flag
		jp z, PostRPGMove              ; if RPGs are not to change their Y coords this iteration of the main loop the jump past the RPG Y update instructions
		
			; if RPG round is above player then move downwards
			push bc                    ; push bc to stack
			ld a, (de)                 ; load RPG y coord into register a
			ld b, a                    ; shift RPG y coord to register b
			ld a, (Ambulance.y)        ; load player y coord into register a
			sub b                      ; subtract b from a
			pop bc                     ; pop bc back to register
			jp m, NextCheck1           ; if RPG is below player then jump to next check
		
			; moves RPG round down
			ld a, (de)                 ; load RPG y coord into register a
			inc a                      ; increment register a (i.e. move RPG round down)
			ld (de), a                 ; set new RPG round y coord
		
			NextCheck1:                ; NextCheck1 marker

			; if RPG round is below player then move upwards
			push bc                    ; push bc to stack
			ld a, (de)                 ; load RPG round y coord into register a
			ld b, a                    ; shift RPG round y coord to register b
			ld a, (Ambulance.y)        ; load player y coord into register a
			sub b                      ; subtract b from a
			pop bc                     ; pop bc back to register
			jp p, NextCheck2           ; if RPG round is above player then to next check
		
			; moves RPG round up
			ld a, (de)                 ; load RPG round y coord into register a
			dec a                      ; decrement register a (i.e. move RPG round up)
			ld (de), a                 ; set new RPG round y coord
			
			NextCheck2:                ; NextCheck2 marker
			
			ld a, $00                  ; load 0 into register a
			ld (RPGYMoveFlag), a       ; reset RPGYMoveFlag
		
		PostRPGMove:                   ; PostRPGMove marker
		
		; write y coord for front half of RPG round to sprite table
		ld a, (de)                     ; load RPG round y coordinate into register a
		ld (hl), a                     ; write RPG round y coord to local sprite table
		
		inc hl                         ; increment hl to point to y coord for back half of RPG round in local sprite table
		
		; write y coord for back half of RPG round to sprite table
		ld (hl), a                     ; write RPG round y coord to local sprite table
		
		inc hl                         ; increment hl to point to next RPG round y coord location in local sprite table
		
		EndRPGYUpdateLoop:             ; EndRPGYUpdateLoop marker
		    pop de                     ; retrieve start of loop de pointer and then increment to next RPG round object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,RPGYUpdateLoop       ; if nz = 1 then jump to RPGYUpdateLoop
	
	ld (SpriteTableYPointer), hl       ; update sprite table Y pointer
	
	ld bc, (NoOfRPGRds)                ; load number of RPG rounds into counter register bc
	ld de, RPGRds				       ; load start address of RPG round objects into register de
	ld hl, (SpriteTableXTPointer)      ; load local sprite table address to write x coord and tile indexes to into register hl
	
	RPGXTUpdateLoop:                   ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndRPGXTUpdateLoop      ; jump to the end of the RPG rounds XT Update loop
		
		inc de                         ; increment de to point to RPG rounds y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndRPGXTUpdateLoop      ; jump to the end of the RPG rounds XT Update loop
		
		inc de                         ; increment de twice to point to RPG rounds x coord
		inc de
		
		; moves RPG round left
		ld a, (de)                     ; load RPG round x coord into register a
		dec a                          ; decrement register a (i.e. move RPG round left)
		ld (de), a                     ; set new RPG round x coord
		
		; set x coord of front half of RPG round
		ld a, (de)                     ; load RPG round x coordinate into register a
		ld (hl), a                     ; write RPG round x coord to local sprite table
		
		; set x coord of back half of RPG round
		inc hl                         ; increment hl twice to point to x coord of back half of RPG round in local sprite table
		inc hl
		add a, 8                       ; add 8 to x coord of front half of RPG round
		ld (hl), a                     ; write RPG round x coord to local sprite table
		
		; set tile index of front half of RPG round
		dec hl                         ; decrement hl to point to RPG round tile index location in local sprite table
		inc de                         ; increment de to point to the RPG round tile index bottom
		ld a, (de)                     ; load RPG round tile index into register a
		ld (hl), a                     ; write RPG round tile index to local sprite table
	
		; set tile index of back half of RPG round
		inc hl                         ; increment hl twice to point to back half of RPG round tile index in local sprite table
		inc hl
		inc de                         ; increment de to point to the RPG round tile index back
		ld a, (de)                     ; load RPG round tile index into register a
		ld (hl), a                     ; write RPG round tile index to local sprite table
		
		inc hl                         ; increment hl to point to next RPG x coord location in local sprite table
		
		EndRPGXTUpdateLoop:
			pop de                     ; retrieve start of loop de pointer and then increment to next RPG object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,RPGXTUpdateLoop      ; if nz = 1 then jump to RPGXTUpdateLoop
	
	ld (SpriteTableXTPointer), hl      ; update sprite table XT pointer
	
	; update IED sprites
    ld bc, (NoOfIEDs)                  ; load number of IEDs into counter register bc
	ld de, IEDs				           ; load start address of IED objects into register de
	ld hl, (SpriteTableYPointer)       ; load local sprite table address to write y coord to into register hl
	
	IEDYUpdateLoop:                    ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndIEDYUpdateLoop       ; jump to the end of the IED round Y Update loop

		inc de                         ; increment de to point to IED round y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndIEDYUpdateLoop       ; jump to the end of the IED round Y Update loop
		
		inc de                         ; increment de to point at IED round y coordinate value
		
		ld a, (de)                     ; load IED round y coordinate into register a
		ld (hl), a                     ; write IED round y coord to local sprite table
		
		inc hl                         ; increment hl to point to next IED round y coord location in local sprite table
		
		EndIEDYUpdateLoop:             ; EndIEDYUpdateLoop marker
		    pop de                     ; retrieve start of loop de pointer and then increment to next IED round object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,IEDYUpdateLoop       ; if nz = 1 then jump to IEDYUpdateLoop
	
	ld (SpriteTableYPointer), hl       ; update sprite table Y pointer
	
	ld bc, (NoOfIEDs)                  ; load number of IED rounds into counter register bc
	ld de, IEDs				           ; load start address of IED round objects into register de
	ld hl, (SpriteTableXTPointer)      ; load local sprite table address to write x coord and tile indexes to into register hl
	
	IEDXTUpdateLoop:                   ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndIEDXTUpdateLoop      ; jump to the end of the IED rounds XT Update loop
		
		inc de                         ; increment de to point to IED rounds y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndIEDXTUpdateLoop      ; jump to the end of the IED rounds XT Update loop
		
		inc de                         ; increment de twice to point to arty rounds x coord
		inc de
		
		ld a, (de)                     ; load IED round x coordinate into register a
		ld (hl), a                     ; write IED round x coord to local sprite table
		
		inc hl                         ; increment hl to point to tile index in local sprite table
		inc de                         ; increment de to point to the IED round tile index bottom

		ld a, (de)                     ; load IED round tile index into register a
		ld (hl), a                     ; write IED round tile index to local sprite table
		
		inc hl                         ; increment hl to point to next IED x coord location in local sprite table
		
		EndIEDXTUpdateLoop:
			pop de                     ; retrieve start of loop de pointer and then increment to next object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,IEDXTUpdateLoop      ; if nz = 1 then jump to IEDXTUpdateLoop
	
	ld (SpriteTableXTPointer), hl      ; update sprite table XT pointer
	
	; update casualty sprites
    ld bc, (NoOfCas)                   ; load number of casualties into counter register bc
	ld de, Casualties			       ; load start address of casualties objects into register de
	ld hl, (SpriteTableYPointer)       ; load local sprite table address to write y coord to into register hl
	
	CasYUpdateLoop:                    ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndCasYUpdateLoop       ; jump to the end of the casualties Y Update loop

		inc de                         ; increment de to point to casualties y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndCasYUpdateLoop       ; jump to the end of the casualties Y Update loop
		
		inc de                         ; increment de to point at casualties y coordinate value
		
		; write y coord for front half of casualties to sprite table
		ld a, (de)                     ; load casualties y coordinate into register a
		ld (hl), a                     ; write casualties y coord to local sprite table
	
		inc hl                         ; increment hl to point to y coord for back half of casualties in local sprite table
		
		; write y coord for back half of casualties to sprite table
		ld (hl), a                     ; write casualties y coord to local sprite table
		
		inc hl                         ; increment hl to point to next casualties y coord location in local sprite table
		
		EndCasYUpdateLoop:             ; EndCasYUpdateLoop marker
		    pop de                     ; retrieve start of loop de pointer and then increment to next arty round object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,CasYUpdateLoop       ; if nz = 1 then jump to CasYUpdateLoop
	
	ld (SpriteTableYPointer), hl       ; update sprite table Y pointer
	
	ld bc, (NoOfCas)                   ; load number of casualties into counter register bc
	ld de, Casualties   		       ; load start address of casualties objects into register de
	ld hl, (SpriteTableXTPointer)      ; load local sprite table address to write x coord and tile indexes to into register hl
	
	CasXTUpdateLoop:                   ; top of loop (analogous to for loop)
		push de                        ; push begining value of de to stack for reference later
		
		push bc                        ; push bc to stack
		call CheckXIndexUpdateLoop     ; check to see if player is in the same x map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndCasXTUpdateLoop      ; jump to the end of the casualties XT Update loop
		
		inc de                         ; increment de to point to casualties y map index
		
		push bc                        ; push bc to stack
		call CheckYIndexUpdateLoop     ; check to see if player is in the same y map index as the sprite
		pop bc                         ; pop bc back to register
		jp nz, EndCasXTUpdateLoop      ; jump to the end of the casualties XT Update loop
		
		inc de                         ; increment de twice to point to casualties x coord
		inc de
		
		; set x coord of front half of casualties
		ld a, (de)                     ; load casualties x coordinate into register a
		ld (hl), a                     ; write casualties x coord to local sprite table
		
		; set x coord of back half of casualties
		inc hl                         ; increment hl twice to point to x coord of back half of casualties in local sprite table
		inc hl
		add a, 8                       ; add 8 to x coord of front half sprite
		ld (hl), a                     ; write casualties x coord to local sprite table
		
		; set tile index of front half of casualties
		dec hl                         ; decrement hl to point to casualties tile index location in local sprite table
		inc de                         ; increment de to point to the casualties tile index front
		ld a, (de)                     ; load casualties tile index into register a
		ld (hl), a                     ; write casualties tile index to local sprite table
		
		; set tile index of back half of casualties
		inc hl                         ; increment hl twice to point to back half of casualties tile index in local sprite table
		inc hl
		inc de                         ; increment de to point to the casualties tile index top
		ld a, (de)                     ; load casualties tile index into register a
		ld (hl), a                     ; write casualties tile index to local sprite table
		
		inc hl                         ; increment hl to point to next casualties x coord location in local sprite table
		
		EndCasXTUpdateLoop:
			pop de                     ; retrieve start of loop de pointer and then increment to next casualties object
			call EndUpdateLoop         ; conduct update loop ending instructions
			jp nz,CasXTUpdateLoop      ; if nz = 1 then jump to CasXTUpdateLoop
	
	ld (SpriteTableXTPointer), hl      ; update sprite table XT pointer
	
	; send updated sprite map to the VDP
    ld a,$00                           ; load $00 into register a
	out (VDPControl),a                 ; output register a to VDP control port
	ld a,SpritemapWrite                ; load sprite map location into register a
	out (VDPControl),a                 ; output register a to VDP control port
	ld hl,SpriteTable                  ; load address of local RAM sprite table into register hl
	ld bc,256                          ; load the size of the local RAM sprite table in bytes to register bc
	call SendToVDPData                 ; send the local RAM sprite table to the VDP sprite table in VRAM
	
	ret

; Control and game mechanics functions
; ====================================

; A function to get input from the player
; ---------------------------------------
; Parameters: nil
; Acknowledgements to Maxim at SMSPower. This was the hardest problem to solve and his
; example source code was central to sorting out how to handle user input. See Maxim's
; ono101 game source code.
GetInputs:                        ; function definition
	in a,(IOPort1)                ; receive input from IOPort1
	cpl                           ; I'm not too clear on how this works
	and P1U|P1D|P1L|P1R|P11|P12   ; logical and with all possible key input signals
	ld b,a                        ; I'm not too clear on how this works
	ld hl,CurrentlyPressedButtons ; shift currently pressed buttons to register hl
	xor (hl)                      ; logical xor on register hl
	ld (hl),b                     ; I'm not too clear on how this works
	and b                         ; I'm not too clear on how this works
	ld (JustPressedButtons),a     ; I'm not too clear on how this works
	in a,(IOPort2)                ; receive input from IOPort2
	cpl                           ; I'm not too clear on how this works
	and ResetButton               ; logical and with reset button input signal
	jp nz,0                       ; I'm not too clear on how this works
	ret                           ; return to last execution

; A function to wait for user key press
; -------------------------------------
; Parameters: nil
WaitForKeyPress:                       ; function definition
	WaitLoop:                          ; top of loop (analogous to while loop)
		call GetInputs                 ; get input from user
		ld a,(CurrentlyPressedButtons) ; load CurrentlyPressedButtons buttons into register a
		and P1U|P1D|P1L|P1R|P11|P12    ; compare CurrentlyPressedButtons to those the user may press for effect
		jp z,WaitLoop                  ; while no buttons pressed jump to top of loop
	
	ret                                ; return to last execution
	
; A function to move player based on user input
; ---------------------------------------------
; Parameters: nil
MovePlayer:
	; conditional statements re: user input
	ld a, (CurrentlyPressedButtons)     ; load currently pressed buttons into register a
	and P1U                             ; if player presses up then
	jp nz, MoveUp                       ; jump to the MoveUp code

	ld a, (CurrentlyPressedButtons)     ; load currently pressed buttons into register a
	and P1D                             ; if player presses down then
	jp nz, MoveDown                     ; jump to the MoveDown code
	
	ld a, (CurrentlyPressedButtons)     ; load currently pressed buttons into register a
	and P1L                             ; if player presses left then
	jp nz, MoveLeft                     ; jump to the MoveLeft code

	ld a, (CurrentlyPressedButtons)     ; load currently pressed buttons into register a
	and P1R                             ; if player presses right then
	jp nz, MoveRight                    ; jump to the MoveRight code
	
	jp EndFunctionMovePlayer            ; if no conditions are met, then jump to end of function to avoid uncommanded movement
	
	; Move player up
	MoveUp:                             ; MoveUp marker
		ld bc, 64                       ; load 64 into counter bc
		ld hl, (CurrentBGMatrixAddress) ; load current CurrentBGMatrixAddress into register hl
		
		; a loop to point to the tile above the player
		MoveUpCheckLoop:
			dec hl
			dec bc
			ld a, b
			or c
			jp nz, MoveUpCheckLoop
		
		; check to see if proposed tile is solid or not
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 71                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 72                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 175                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 176                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 177                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 178                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 179                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 180                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 181                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 182                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 183                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 184                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 185                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 186                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 187                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 188                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 189                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 190                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 191                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		
		call TrackMvmtPSG               ; trigger track vehicle movement sound
		
		ld a, PlayerUp                  ; load up direction value into register a
		ld (DirectionOfTravel), a       ; shift direction of travel value into variable

		call ToggleWheelsTurningFlag; toggle the flag controlling wheel turning animation
		
		ld a, (Ambulance.y)             ; load ambulance's y coord into register a
		dec a                           ; decrement register a
		ld (Ambulance.y), a             ; load register a into ambulance's y coord
		cp TopOfMap                     ; check to see if ambulance is at the top of the current map
		jp z, DecrementMapYIndex        ; if so jump to decrementing the map y index
		jp EndFunctionMovePlayer        ; if not jump to end of function
	
	; Move player down
	MoveDown:                           ; MoveDown marker
		ld bc, 64                       ; load 64 into counter bc
		ld hl, (CurrentBGMatrixAddress) ; load current CurrentBGMatrixAddress into register hl
		
		; a loop to point to the tile below the player
		MoveDownCheckLoop:
			inc hl
			dec bc
			ld a, b
			or c
			jp nz, MoveDownCheckLoop
		
		; check to see if proposed tile is solid or not
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 71                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 72                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 175                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 176                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 177                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 178                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 179                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 180                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 181                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 182                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 183                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 184                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 185                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 186                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 187                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 188                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 189                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 190                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 191                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		
		call TrackMvmtPSG               ; trigger track vehicle movement sound
		
		ld a, PlayerDown                ; load down direction value into register a
		ld (DirectionOfTravel), a       ; shift direction of travel value into variable
	
		call ToggleWheelsTurningFlag    ; toggle the flag controlling wheel turning animation
	
		ld a, (Ambulance.y)             ; load ambulance's y coord into register a
		inc a                           ; decrement register a
		ld (Ambulance.y), a             ; load register a into ambulance's y coord
		cp BotOfMap                     ; check to see if ambulance is at the bottom of the current map
		jp z, IncrementMapYIndex        ; if so jump to incrementing the map y index
		jp EndFunctionMovePlayer        ; if not jump to end of function
	
	; Move player left
	MoveLeft:                           ; MoveLeft marker
		ld hl, (CurrentBGMatrixAddress) ; load current CurrentBGMatrixAddress into register hl
		
		; manipulation of hl is not required to point to tile to the left
		
		; check to see if proposed tile is solid or not
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 71                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 72                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 175                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 176                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 177                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 178                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 179                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 180                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 181                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 182                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 183                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 184                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 185                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 186                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 187                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 188                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 189                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 190                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 191                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		
		call TrackMvmtPSG               ; trigger track vehicle movement sound
		
		ld a, PlayerLeft                ; load left direction value into register a
		ld (DirectionOfTravel), a       ; shift direction of travel value into variable
		
		call ToggleWheelsTurningFlag    ; toggle the flag controlling wheel turning animation
		
		ld a, (Ambulance.x)             ; load ambulance's x coord into register a
		dec a                           ; decrement register a
		ld (Ambulance.x), a             ; load register a into ambulance's x coord
		cp 0                            ; check to see if ambulance is at the left edge of the current map
		jp z, DecrementMapXIndex        ; if so jump to decrementing the map x index
		jp EndFunctionMovePlayer        ; if not jump to end of function
	
	; Move player right
	MoveRight:                          ; MoveRight marker
		ld hl, (CurrentBGMatrixAddress) ; load current CurrentBGMatrixAddress into register hl

		inc hl                          ; increment hl twice to point at tile to the right
		inc hl
		
		; check to see if proposed tile is solid or not
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 71                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 72                           ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 175                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 176                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 177                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 178                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 179                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 180                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 181                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 182                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 183                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 184                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 185                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 186                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 187                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		cp 188                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 189                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 190                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		ld a, (hl)                      ; load value at location hl in BGmatrix into register de
		cp 191                          ; if register a contains solid tile index then
		jp z, EndFunctionMovePlayer     ; jump to end of function
		
		call TrackMvmtPSG               ; trigger track vehicle movement sound
		
		ld a, PlayerRight               ; load left direction value into register a
		ld (DirectionOfTravel), a       ; shift direction of travel value into variable
		
		call ToggleWheelsTurningFlag    ; toggle the flag controlling wheel turning animation
		
		ld a, (Ambulance.x)             ; load ambulance's x coord into register a
		inc a                           ; decrement register a
		ld (Ambulance.x), a             ; load register a into ambulance's x coord
		cp 255                          ; check to see if ambulance is at the right edge of the current map
		jp z, IncrementMapXIndex        ; if so jump to decrementing the map x index
		jp EndFunctionMovePlayer        ; if not jump to end of function
	
	; Decrement the Y Map index
	DecrementMapYIndex:                 ; IncrementMapYIndex marker
		ld a, BotOfMap                  ; load BotOfMap into register a
		ld (Ambulance.y), a             ; reset Ambulance's y coord to BotOfMap at top of screen
		ld a, $01                       ; load $01 into register a
		ld (MapChangeFlag), a           ; set MapChangeFlag to 1 to trigger redraw
		ld a, (Ambulance.map_y)         ; load Ambulance's y map index into register a
		cp 0                            ; if map y index is 0
		jp z, SetYIndex2                ; set y index to 2
		dec a                           ; increment register a
		ld (Ambulance.map_y), a         ; load register a into Ambulance's y map index
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; Increment the Y Map index
	IncrementMapYIndex:                 ; IncrementMapYIndex marker
		ld a, TopOfMap                  ; load TopOfMap into register a
		ld (Ambulance.y), a             ; reset Ambulance's y coord to 0 at bottom of screen
		ld a, $01                       ; load $01 into register a
		ld (MapChangeFlag), a           ; set MapChangeFlag to 1 to trigger redraw
		ld a, (Ambulance.map_y)         ; load Ambulance's y map index into register a
		cp 2                            ; if map y index is 2
		jp z, SetYIndex0                ; set y index to 0
		inc a                           ; increment register a
		ld (Ambulance.map_y), a         ; load register a into Ambulance's y map index
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; If Ambulance is at the top of map index then wrap him down to 2
	SetYIndex2:                         ; SetYIndex2 marker
		ld a, 2                         ; load 2 into register a
		ld (Ambulance.map_y), a         ; set y index to 2
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; If Ambulance is at the bottom of map index then wrap him up to 0
	SetYIndex0:                         ; SetYIndex0 marker
		ld a, 0                         ; load 0 into register a
		ld (Ambulance.map_y), a         ; set y index to 0
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; Decrement the X Map index
	DecrementMapXIndex:                 ; IncrementMapXIndex marker
		ld a, $01                       ; load $01 into register a
		ld (MapChangeFlag), a           ; set MapChangeFlag to 1 to trigger redraw
		ld a, (Ambulance.map_x)         ; load Ambulance's x map index into register a
		cp 0                            ; if map x index is 0
		jp z, SetXIndex5                ; set x index to 5
		dec a                           ; increment register a
		ld (Ambulance.map_x), a         ; load register a into Ambulance's x map index
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; Increment the X Map index
	IncrementMapXIndex:                 ; IncrementMapXIndex marker
		ld a, $01                       ; load $01 into register a
		ld (MapChangeFlag), a           ; set MapChangeFlag to 1 to trigger redraw
		ld a, (Ambulance.map_x)         ; load Ambulance's x map index into register a
		cp 5                            ; if map x index is 5
		jp z, SetXIndex0                ; set x index to 0
		inc a                           ; increment register a
		ld (Ambulance.map_x), a         ; load register a into Ambulance's y map index
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; If Ambulance is at the left edge of map index then wrap him up to 5
	SetXIndex5:                         ; SetXIndex3 marker
		ld a, 5                         ; load 5 into register a
		ld (Ambulance.map_x), a         ; set x index to 5
		jp EndFunctionMovePlayer        ; jump to end of function
	
	; If Ambulance is at the right edge of map index then wrap him down to 0
	SetXIndex0:                         ; SetXIndex0 marker
		ld a, 0                         ; load 0 into register a
		ld (Ambulance.map_x), a         ; set x index to 0
		jp EndFunctionMovePlayer        ; jump to end of function
	
	EndFunctionMovePlayer:              ; End of function marker
	
	ret

; A function to send data to the BGM
; ----------------------------------
; Parameters: hl = start address of data, bc = end address of
;             data - start address of data
SendToBGMatrix:                  ; function definition
	ld de, BackgroundMatrix      ; load start address of the BGM into register de

    SendToBGMatrixLoop:          ; top of loop (analogous to for loop)
        ld a,(hl)                ; load data at address stored in hl into register a
		ld (de), a               ; move data in BGMatrix
        inc hl                   ; add one to hl so it points to the next data byte
		inc de                   ; add one to de so it points to the next empty byte
        dec bc                   ; decrement counter
        ld a,b                   ; load high byte (b) from counter into register a
        or c                     ; compare a and c; if both are zero then sets the nz flag
        jp nz,SendToBGMatrixLoop ; if nz = 1 then jump to SendToBGMatrixLoop
	
	ret                          ; return to last execution

; A function to calculate player's Current Column and Row
; -------------------------------------------------------
; Parameters: nil
CalcPlayerCurrentColumnRow:            ; function definition
	; Determine current column
	ld bc, 32                          ; load number of columns into counter bc
	ld hl, 1                           ; load 1st column value into register d
	ld e, 1                            ; load 1st x coord value to be tested into register e
	
	ColumnDeterminationLoop:           ; top of loop (analogous to for loop)
		ld a, (Ambulance.x)            ; load player's current x coord into register a
		
		; compare player's current x coord with the 8 pixels in the column currently under consideration
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual x coord
		jp z, SetColumn                ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		inc hl                         ; increment the column value
		dec bc                         ; decrement counter
		ld a, b                        ; load high byte of bc into a
		or c                           ; compare c and a
		jp nz, ColumnDeterminationLoop ; if counter not zero jump to top of loop
	
	SetColumn:                         ; set column marker
		ld (CurrentColumn), hl         ; set current column
	
	; Determine current row
	ld bc, 23                          ; load number of columns into counter bc
	ld hl, 1                           ; load 1st column value into register d
	ld e, 1                            ; load 1st y coord value to be tested into register e
	
	RowDeterminationLoop:              ; top of loop (analogous to for loop)
		ld a, (Ambulance.y)            ; load player's current y coord into register a
		
		; compare player's current y coord with the 8 pixels in the column currently under consideration
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		cp e                           ; compare e with player's actual y coord
		jp z, SetRow                   ; if true then jump to SetColumn
		inc e                          ; increment to next pixel
		
		inc hl                         ; increment the column value
		dec bc                         ; decrement counter
		ld a, b                        ; load high byte of bc into a
		or c                           ; compare c and a
		jp nz, RowDeterminationLoop    ; if counter not zero jump to top of loop
	
	SetRow:                            ; set row marker
		ld (CurrentRow), hl            ; set current row
	
	; Determine current BackgroundMatrix address
	ld de, 64                                   ; load 64 to register de
	ld hl, BackgroundMatrix                     ; load starting address of the BackgroundMatrix into register hl
	ld bc, (CurrentRow)                         ; load current number of rows into counter bc
	dec bc                                      ; decrement bc to row above current
	ld a, b                                     ; if bc = 0 then
	or c
	jp z, ColumnAddressComponent                ; then jump to ColumnAddressComponent
	
	BGAddressDeterminationLOOP1:                ; top of loop (analogous to for loop)
		add hl, de                              ; add 64 to register hl
		
		dec bc                                  ; decrement counter
		ld a, b                                 ; load high byte of bc into a
		or c                                    ; compare c and a
		jp nz, BGAddressDeterminationLOOP1      ; if counter not zero jump to top of loop
	
	ld (CurrentBGMatrixAddress), hl             ; set CurrentBGMatrixAddress based on row component
	
	ColumnAddressComponent:                     ; ColumnAddressComponent marker
	
	ld bc, (CurrentColumn)                      ; load column number into counter
	dec bc                                      ; decrement bc to column to left of current
	ld a, b                                     ; if bc = 0 then
	or c
	jp z, EndFunctionCalcPlayerCurrentColumnRow ; then jump to EndFunctionCalcPlayerCurrentColumnRow
	
	BGAddressDeterminationLOOP2:                ; top of loop (analogous to for loop)
		inc hl                                  ; increment hl
		inc hl
	
		dec bc                                  ; decrement counter
		ld a, b                                 ; load high byte of bc into a
		or c                                    ; compare c and a
		jp nz, BGAddressDeterminationLOOP2      ; if counter not zero jump to top of loop
	
	EndFunctionCalcPlayerCurrentColumnRow:      ; EndFunctionCalcPlayerCurrentColumnRow marker
	
	ld (CurrentBGMatrixAddress), hl             ; set CurrentBGMatrixAddress
	
	ret                                         ; return to last execution

; A function to handle collision events
; -------------------------------------
; Parameters: nil
CollisionHandler:                     ; function definition
	in a,(VDPControl)                 ; read the output of the VDPControl port to detect sprite collision signal
	and COL                           ; conduct logical and to isolate collision signal
	jp z, EndFunctionCollisionHandler ; if no collision detected, jump to End of function
	
	; handle casualty collisions
	ld hl, Casualties                 ; load address of casulaty objects into register hl
	ld bc, (NoOfCas)                  ; load number of casualties saved into counter bc
	
	CasCollisionLoop:                 ; top of loop (analogous to for loop)
		push hl                       ; push begining value of hl to stack for reference later
		
		; test to see if cas is in same map x index
		push bc                       ; push bc to stack
		call CheckXIndexCollisionLoop ; check to see if sprite and player have the same x map index
		pop bc                        ; pop bc back to register
		jp nz, EndCasCollisionLoop    ; jump to the end of the cas XT Update loop
		
		inc hl                        ; increment hl to point to cas y map index
		
		; test to see if cas is in same map y index
		push bc                       ; push bc to stack
		call CheckYIndexCollisionLoop ; check to see if sprite and player have the same y map index
		pop bc                        ; pop bc back to register
		jp nz, EndCasCollisionLoop    ; jump to the end of the cas XT Update loop
		
		inc hl                        ; increment hl to point to cas y coord
		
		; test to see if player is below the y upper limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load cas y coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define top of collision box
		ld b, a                       ; shift top of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndCasCollisionLoop     ; if player is above cas collision box jump to the end of the loop
		
		; test to see if player is above the y lower limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load cas y coord into register a
		add a, 8                      ; add 8 pixels to a to define bottom of collision box
		ld b, a                       ; shift bottom of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndCasCollisionLoop     ; if player is below cas collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to cas x coord
		
		; test to see if player is to the right of the left limit of the collition box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load cas x coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define left of collision box
		ld b, a                       ; shift left of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndCasCollisionLoop     ; if player is left of the cas collision box jump to the end of the loop
		
		; test to see if player is to the left of the right limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load cas x coord into register a
		add a, 8                      ; add 8 pixels to a to define right of collision box
		ld b, a                       ; shift right of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndCasCollisionLoop     ; if player is right of the cas collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to cas tile index
		
		; test to see if cas has been blanked previously
		ld a, (hl)                    ; load cas tile index into register a
		cp $00                        ; compare cas tile index with blank index
		jp z, EndCasCollisionLoop     ; if cas is blanked then jump to the end of the loop
		
		ld (hl), $00                  ; set the cas tile 1 to blank so it dissappears from the screen
		inc hl                        ; increment hl to point to next cas tile index
		ld (hl), $00                  ; set the cas tile 2 to blank so it disappears form the screen
		
		; increment cas counter
		push bc                       ; push bc to stack
		ld bc, (CasualtiesSaved)      ; load current cas saved player has into bc
		inc bc                        ; increment current cas saved player has
		ld (CasualtiesSaved), bc      ; place players cas saved back in memory
		pop bc                        ; pop bc back to register
		
		ld a, $01                     ; load $01 into register a
		ld (MapChangeFlag), a         ; set MapChangeFlag to 1 to trigger redraw
		
		; test to see if player has 10 cas saved. If so then draw the you win screen
		push bc                       ; push bc to stack
		ld bc, (CasualtiesSaved)      ; load current cas saved player has into bc
		ld a, c                       ; load low byte of current cas saved into register a
		cp MaxCasSaved                ; if player does not have all the casualties
		pop bc                        ; pop bc back to register
		jp nz, EndCasCollisionLoop    ; then jump to the end of the cas collision loop
		
		call DrawWinScreen            ; draw win screen if player has collected all casualties
		
		EndCasCollisionLoop:
			pop hl                    ; restore previous value of hl
			call EndCollisionLoop     ; perform operations required to complete the loop
			jp nz, CasCollisionLoop   ; if counter is not zero then jump to top of loop
	
	; handle arty round collisions
	ld hl, ArtyRds                    ; load address of arty round objects into register hl
	ld bc, (NoOfArtyRds)              ; load number of arty rounds into counter bc
	
	ArtyCollisionLoop:                ; top of loop (analogous to for loop)
		push hl                       ; push begining value of hl to stack for reference later
		
		; test to see if round is in same map x index
		push bc                       ; push bc to stack
		call CheckXIndexCollisionLoop ; check to see if sprite and player have the same x map index
		pop bc                        ; pop bc back to register
		jp nz, EndArtyCollisionLoop   ; jump to the end of the arty collision loop
		
		inc hl                        ; increment hl to point to round y map index
		
		; test to see if round is in same map y index
		push bc                       ; push bc to stack
		call CheckYIndexCollisionLoop ; check to see if sprite and player have the same y map index
		pop bc                        ; pop bc back to register
		jp nz, EndArtyCollisionLoop   ; jump to the end of the arty collision loop
		
		inc hl                        ; increment hl to point to round y coord
		
		; test to see if player is below the y upper limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round y coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define top of collision box
		ld b, a                       ; shift top of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		ld (Explosion.y), a           ; set explosion y coord to that of the round in case there is an impact
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndArtyCollisionLoop    ; if player is above collision box jump to the end of the loop
		
		; test to see if player is above the y lower limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round y coord into register a
		add a, 8                      ; add 8 pixels to a to define bottom of collision box
		ld b, a                       ; shift bottom of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndArtyCollisionLoop    ; if player is below collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to arty x coord
		
		; test to see if player is to the right of the left limit of the collition box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round x coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define left of collision box
		ld b, a                       ; shift left of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		ld (Explosion.x), a           ; set explosion x coord to that of the round in case there is an impact
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndArtyCollisionLoop    ; if player is left of the collision box jump to the end of the loop
		
		; test to see if player is to the left of the right limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round x coord into register a
		add a, 8                      ; add 8 pixels to a to define right of collision box
		ld b, a                       ; shift right of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndArtyCollisionLoop    ; if player is right of the collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to round tile index
		
		; test to see if round has been blanked previously
		ld a, (hl)                    ; load round tile index into register a
		cp $00                        ; compare round tile index with blank index
		jp z, EndArtyCollisionLoop    ; if round is blanked then jump to the end of the loop
		
		ld (hl), $00                  ; set the round tile 1 to blank so it dissappears from the screen
		inc hl                        ; increment hl to point to next cas tile index
		ld (hl), $00                  ; set the round tile 2 to blank so it disappears from the screen
		
		; set explosion sprite to show up on next update sprite function call
		ld a, 84                      ; load explosion tile number into register a
		ld (Explosion.tile), a        ; set explosion sprite tile number
		
		; set duration that explosion sprite remains on screen
		ld bc, ExplosionDuration      ; load explosion duration into register bc
		ld (ExplosionCounter), bc     ; set explosion counter to zero
		
		; decrement life counter
		push bc                       ; push bc to stack
		ld bc, (Lives)                ; load current lives player has into bc
		dec bc                        ; decrement current lives player has
		ld (Lives), bc                ; place players lives back in memory
		pop bc                        ; pop bc back to register
		
		ld a, $01                     ; load $01 into register a
		ld (MapChangeFlag), a         ; set MapChangeFlag to 1 to trigger redraw
		
		; test to see if player 0 lives. If so then draw the game over screen
		push bc                       ; push bc to stack
		ld bc, (Lives)                ; load current lives player has into bc
		ld a, c                       ; load low byte of current lives into register a
		cp 0                          ; if player still has lives
		pop bc                        ; pop bc back to register
		jp nz, EndArtyCollisionLoop   ; then jump to the end of the collision loop
		
		call DrawGameOverScreen       ; draw game over screen if player has lost all lives
		
		EndArtyCollisionLoop:
			pop hl                    ; restore previous value of hl
			call EndCollisionLoop     ; perform operations required to complete the loop
			jp nz, ArtyCollisionLoop  ; if counter is not zero then jump to top of loop
	
	; handle IED round collisions
	ld hl, IEDs                       ; load address of IED round objects into register hl
	ld bc, (NoOfIEDs)                 ; load number of IED rounds into counter bc
	
	IEDCollisionLoop:                 ; top of loop (analogous to for loop)
		push hl                       ; push begining value of hl to stack for reference later
		
		; test to see if round is in same map x index
		push bc                       ; push bc to stack
		call CheckXIndexCollisionLoop ; check to see if sprite and player have the same x map indexn
		pop bc                        ; pop bc back to register
		jp nz, EndIEDCollisionLoop    ; jump to the end of the arty collision loop
		
		inc hl                        ; increment hl to point to round y map index
		
		; test to see if round is in same map y index
		push bc                       ; push bc to stack
		call CheckYIndexCollisionLoop ; check to see if sprite and player have the same y map index
		pop bc                        ; pop bc back to register
		jp nz, EndIEDCollisionLoop    ; jump to the end of the arty collision loop
		
		inc hl                        ; increment hl to point to round y coord
		
		; test to see if player is below the y upper limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round y coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define top of collision box
		ld b, a                       ; shift top of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		ld (Explosion.y), a           ; set explosion y coord to that of the round in case there is an impact
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndIEDCollisionLoop     ; if player is above collision box jump to the end of the loop
		
		; test to see if player is above the y lower limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round y coord into register a
		add a, 8                      ; add 8 pixels to a to define bottom of collision box
		ld b, a                       ; shift bottom of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndIEDCollisionLoop     ; if player is below collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to arty x coord
		
		; test to see if player is to the right of the left limit of the collition box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round x coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define left of collision box
		ld b, a                       ; shift left of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		ld (Explosion.x), a           ; set explosion x coord to that of the round in case there is an impact
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndIEDCollisionLoop     ; if player is left of the collision box jump to the end of the loop
		
		; test to see if player is to the left of the right limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round x coord into register a
		add a, 8                      ; add 8 pixels to a to define right of collision box
		ld b, a                       ; shift right of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndIEDCollisionLoop     ; if player is right of the collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to round tile index
		
		; test to see if round has been blanked previously
		ld a, (hl)                    ; load round tile index into register a
		cp $00                        ; compare round tile index with blank index
		jp z, EndIEDCollisionLoop     ; if round is blanked then jump to the end of the loop
		
		ld (hl), $00                  ; set the round tile 1 to blank so it dissappears from the screen
		inc hl                        ; increment hl to point to next cas tile index
		ld (hl), $00                  ; set the round tile 2 to blank so it disappears from the screen
		
		; set explosion sprite to show up on next update sprite function call
		ld a, 84                      ; load explosion tile number into register a
		ld (Explosion.tile), a        ; set explosion sprite tile number
		
		; set duration that explosion sprite remains on screen
		ld bc, ExplosionDuration      ; load explosion duration into register bc
		ld (ExplosionCounter), bc     ; set explosion counter to zero
		
		; decrement life counter
		push bc                       ; push bc to stack
		ld bc, (Lives)                ; load current lives player has into bc
		dec bc                        ; decrement current lives player has
		ld (Lives), bc                ; place players lives back in memory
		pop bc                        ; pop bc back to register
		
		ld a, $01                     ; load $01 into register a
		ld (MapChangeFlag), a         ; set MapChangeFlag to 1 to trigger redraw
		
		; test to see if player 0 lives. If so then draw the game over screen
		push bc                       ; push bc to stack
		ld bc, (Lives)                ; load current lives player has into bc
		ld a, c                       ; load low byte of current lives into register a
		cp 0                          ; if player still has lives
		pop bc                        ; pop bc back to register
		jp nz, EndIEDCollisionLoop    ; then jump to the end of the collision loop
		
		call DrawGameOverScreen       ; draw game over screen if player has lost all lives
		
		EndIEDCollisionLoop:
			pop hl                    ; restore previous value of hl
			call EndCollisionLoop     ; perform operations required to complete the loop
			jp nz, IEDCollisionLoop   ; if counter is not zero then jump to top of loop
	
	; handle RPG round collisions
	ld hl, RPGRds                     ; load address of RPG round objects into register hl
	ld bc, (NoOfRPGRds)               ; load number of RPG rounds into counter bc
	
	RPGCollisionLoop:                 ; top of loop (analogous to for loop)
		push hl                       ; push begining value of hl to stack for reference later
		
		; test to see if round is in same map x index
		push bc                       ; push bc to stack
		call CheckXIndexCollisionLoop ; check to see if sprite and player have the same x map index
		pop bc                        ; pop bc back to register
		jp nz, EndRPGCollisionLoop    ; jump to the end of the arty collision loop
		
		inc hl                        ; increment hl to point to round y map index
		
		; test to see if round is in same map y index
		push bc                       ; push bc to stack
		call CheckYIndexCollisionLoop ; check to see if sprite and player have the same y map index
		pop bc                        ; pop bc back to register
		jp nz, EndRPGCollisionLoop    ; jump to the end of the arty collision loop
		
		inc hl                        ; increment hl to point to round y coord
		
		; test to see if player is below the y upper limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round y coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define top of collision box
		ld b, a                       ; shift top of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		ld (Explosion.y), a           ; set explosion y coord to that of the round in case there is an impact
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndRPGCollisionLoop     ; if player is above collision box jump to the end of the loop
		
		; test to see if player is above the y lower limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round y coord into register a
		add a, 8                      ; add 8 pixels to a to define bottom of collision box
		ld b, a                       ; shift bottom of collision box to register b
		ld a, (Ambulance.y)           ; load player's current y coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndRPGCollisionLoop     ; if player is below collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to arty x coord
		
		; test to see if player is to the right of the left limit of the collition box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round x coord into register a
		ld b, 8                       ; load 8 pixels into register b
		sub b                         ; substract b from a to define left of collision box
		ld b, a                       ; shift left of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		ld (Explosion.x), a           ; set explosion x coord to that of the round in case there is an impact
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp m, EndRPGCollisionLoop     ; if player is left of the collision box jump to the end of the loop
		
		; test to see if player is to the left of the right limit of the collision box
		push bc                       ; push bc to stack
		ld a, (hl)                    ; load round x coord into register a
		add a, 8                      ; add 8 pixels to a to define right of collision box
		ld b, a                       ; shift right of collision box to register b
		ld a, (Ambulance.x)           ; load player's current x coord into register a
		sub b                         ; subtract b from a
		pop bc                        ; pop bc back to register
		jp p, EndRPGCollisionLoop     ; if player is right of the collision box jump to the end of the loop
		
		inc hl                        ; increment hl to point to round tile index
		
		; test to see if round has been blanked previously
		ld a, (hl)                    ; load round tile index into register a
		cp $00                        ; compare round tile index with blank index
		jp z, EndRPGCollisionLoop     ; if round is blanked then jump to the end of the loop
		
		ld (hl), $00                  ; set the round tile 1 to blank so it dissappears from the screen
		inc hl                        ; increment hl to point to next cas tile index
		ld (hl), $00                  ; set the round tile 2 to blank so it disappears from the screen
		
		; set explosion sprite to show up on next update sprite function call
		ld a, 84                      ; load explosion tile number into register a
		ld (Explosion.tile), a        ; set explosion sprite tile number
		
		; set duration that explosion sprite remains on screen
		ld bc, ExplosionDuration      ; load explosion duration into register bc
		ld (ExplosionCounter), bc     ; set explosion counter to zero
		
		; decrement life counter
		push bc                       ; push bc to stack
		ld bc, (Lives)                ; load current lives player has into bc
		dec bc                        ; decrement current lives player has
		ld (Lives), bc                ; place players lives back in memory
		pop bc                        ; pop bc back to register
		
		ld a, $01                     ; load $01 into register a
		ld (MapChangeFlag), a         ; set MapChangeFlag to 1 to trigger redraw
		
		; test to see if player 0 lives. If so then draw the game over screen
		push bc                       ; push bc to stack
		ld bc, (Lives)                ; load current lives player has into bc
		ld a, c                       ; load low byte of current lives into register a
		cp 0                          ; if player still has lives
		pop bc                        ; pop bc back to register
		jp nz, EndRPGCollisionLoop    ; then jump to the end of the collision loop
		
		call DrawGameOverScreen       ; draw game over screen if player has lost all lives
		
		EndRPGCollisionLoop:
			pop hl                    ; restore previous value of hl
			call EndCollisionLoop     ; perform operations required to complete the loop
			jp nz, RPGCollisionLoop   ; if counter is not zero then jump to top of loop
	
	EndFunctionCollisionHandler:      ; EndFunctionCollisionHandler marker
	
	ret